#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Related Certificates for Use in Multiple Authentications within a Protocol
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9763.txt
#

from pyasn1.type import char
from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import rfc5751
from pyasn1_alt_modules import rfc6019
from pyasn1_alt_modules import opentypemap

certificateExtensionsMap = opentypemap.get('certificateExtensionsMap')

certificateAttributesMap = opentypemap.get('certificateAttributesMap')

MAX = float('inf')


# Imports from RFC 5280

id_pe = rfc5280.id_pe

Attribute = rfc5280.Attribute

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier



# Imports from RFC 5652

IssuerAndSerialNumber = rfc5652.IssuerAndSerialNumber


# Imports from RFC 5751

id_aa = rfc5751.id_aa


# Imports from RFC 6019

BinaryTime = rfc6019.BinaryTime


# relatedCertificate Extension

id_pe_relatedCert = id_pe + (36, )


class DigestAlgorithmIdentifier(AlgorithmIdentifier):
    pass


class RelatedCertificate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('hashAlgorithm', DigestAlgorithmIdentifier()),
        namedtype.NamedType('hashValue', univ.OctetString())
    )


# relatedCertRequest Attribute

id_aa_relatedCertRequest = id_aa + (60, )


class URI(char.IA5String):
    pass


class UniformResourceIdentifiers(univ.SequenceOf):
    componentType = URI()
    subtypeSpec=constraint.ValueSizeConstraint(1, MAX)


class RequesterCertificate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('certID', IssuerAndSerialNumber()),
        namedtype.NamedType('requestTime', BinaryTime()),
        namedtype.NamedType('locationInfo', UniformResourceIdentifiers()),
        namedtype.NamedType('signature', univ.BitString())
    )


# Update the Certificate Extension Map and the Certificate Attribute Map

_certificateAttributesMapUpdate = {
    id_aa_relatedCertRequest: RequesterCertificate(),
}

certificateAttributesMap.update(_certificateAttributesMapUpdate)


_certificateExtensionsMapUpdate = {
    id_pe_relatedCert: RelatedCertificate(),
}

certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# CMS Encrypted Key Package Content Type
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc6032.txt
#

from pyasn1.type import namedtype
from pyasn1.type import tag
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import rfc5083
from pyasn1_alt_modules import opentypemap

cmsContentTypesMap = opentypemap.get('cmsContentTypesMap')

cmsAttributesMap = opentypemap.get('cmsAttributesMap')


# Content Decryption Key Identifier attribute

id_aa_KP_contentDecryptKeyID = univ.ObjectIdentifier('2.16.840.1.101.2.1.5.66')

class ContentDecryptKeyID(univ.OctetString):
    pass

aa_content_decrypt_key_identifier = rfc5652.Attribute()
aa_content_decrypt_key_identifier['attrType'] = id_aa_KP_contentDecryptKeyID
aa_content_decrypt_key_identifier['attrValues'][0] = ContentDecryptKeyID()


# Encrypted Key Package Content Type

id_ct_KP_encryptedKeyPkg = univ.ObjectIdentifier('2.16.840.1.101.2.1.2.78.2')

class EncryptedKeyPackage(univ.Choice):
    pass

EncryptedKeyPackage.componentType = namedtype.NamedTypes(
    namedtype.NamedType('encrypted', rfc5652.EncryptedData()),
    namedtype.NamedType('enveloped', rfc5652.EnvelopedData().subtype(
        implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
    namedtype.NamedType('authEnveloped', rfc5083.AuthEnvelopedData().subtype(
        implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)))
)


# Update the CMS Attributes Map

_cmsAttributesMapUpdate = {
    id_aa_KP_contentDecryptKeyID: ContentDecryptKeyID(),
}

cmsAttributesMap.update(_cmsAttributesMapUpdate)


# Update the CMS Content Types Map
_cmsContentTypesMapUpdate = {
    id_ct_KP_encryptedKeyPkg: EncryptedKeyPackage(),
}

cmsContentTypesMap.update(_cmsContentTypesMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Using SCVP to convey Long-Term Evidence Records
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5276.txt
#

from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import namedval
from pyasn1.type import univ

from pyasn1_alt_modules import rfc4998
from pyasn1_alt_modules import rfc5055
from pyasn1_alt_modules import opentypemap

scvpWantBackMap = opentypemap.get('scvpWantBackMap')

MAX = float('inf')


# Imports from RFC 4998 and RFC 5055

EvidenceRecord = rfc4998.EvidenceRecord

CertBundle = rfc5055.CertBundle


# Long-Term Evidence Records in SCVP

class EvidenceRecordWantBack(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('targetWantBack', univ.ObjectIdentifier()),
        namedtype.OptionalNamedType('evidenceRecord', EvidenceRecord())
    )


class EvidenceRecordWantBacks(univ.SequenceOf):
    componentType = EvidenceRecordWantBack()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class EvidenceRecords(univ.SequenceOf):
    componentType = EvidenceRecord()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


id_swb = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 18))

id_swb_partial_cert_path = id_swb + (15,)

id_swb_ers_pkc_cert = id_swb + (16,)

id_swb_ers_best_cert_path = id_swb + (17,)

id_swb_ers_partial_cert_path = id_swb + (18,)

id_swb_ers_revocation_info = id_swb + (19,)

id_swb_ers_all = id_swb + (20,)


# Update the SCVP Want Back Map

_scvpWantBackMapUpdate = {
    id_swb_partial_cert_path: CertBundle(),
    id_swb_ers_pkc_cert: EvidenceRecord(),
    id_swb_ers_best_cert_path: EvidenceRecord(),
    id_swb_ers_partial_cert_path: EvidenceRecord(),
    id_swb_ers_revocation_info: EvidenceRecords(),
    id_swb_ers_all: EvidenceRecordWantBacks(),
}

scvpWantBackMap.update(_scvpWantBackMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc8209


class CertificateTestCase(unittest.TestCase):
    cert_pem_text = """\
MIIBiDCCAS+gAwIBAgIEAk3WfDAKBggqhkjOPQQDAjAaMRgwFgYDVQQDDA9ST1VU
RVItMDAwMEZCRjAwHhcNMTcwMTAxMDUwMDAwWhcNMTgwNzAxMDUwMDAwWjAaMRgw
FgYDVQQDDA9ST1VURVItMDAwMEZCRjAwWTATBgcqhkjOPQIBBggqhkjOPQMBBwNC
AARzkbq7kqDLO+EOWbGev/shTgSpHgy6GxOafTjZD3flWqBbjmlWeOD6FpBLVdnU
9cDfxYiV7lC8T3XSBaJb02/1o2MwYTALBgNVHQ8EBAMCB4AwHQYDVR0OBBYEFKtN
kQ9VyucaIV7zyv46zEW17sFUMBMGA1UdJQQMMAoGCCsGAQUFBwMeMB4GCCsGAQUF
BwEIAQH/BA8wDaAHMAUCAwD78KECBQAwCgYIKoZIzj0EAwIDRwAwRAIgB7e0al+k
8cxoNjkDpIPsfIAC0vYInUay7Cp75pKzb7ECIACRBUqh9bAYnSck6LQi/dEc8D2x
OCRdZCk1KI3uDDgp
"""

    def setUp(self):
        self.asn1Spec = rfc5280.Certificate()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.cert_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        extn_list = []

        for extn in asn1Object['tbsCertificate']['extensions']:
            extn_list.append(extn['extnID'])
            if extn['extnID'] in rfc5280.certificateExtensionsMap.keys():
                extnValue, rest = der_decoder(
                    extn['extnValue'],
                    asn1Spec=rfc5280.certificateExtensionsMap[extn['extnID']])

                self.assertEqual(extn['extnValue'], der_encoder(extnValue))

                if extn['extnID'] == rfc5280.id_ce_extKeyUsage:
                    self.assertIn(rfc8209.id_kp_bgpsec_router, extnValue)

        self.assertIn(rfc5280.id_ce_extKeyUsage, extn_list)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2023-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# PkiPath as used in TLS Extensions
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc3546.txt
#

from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280


# PkiPath ::= SEQUENCE OF Certificate

class PkiPath(univ.SequenceOf):
    componentType = rfc5280.Certificate()



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#

import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1.type import univ

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc4010
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import rfc5751


class EnvelopedDataTestCase(unittest.TestCase):
    env_data_pem_text = """\
MIIFewYJKoZIhvcNAQcDoIIFbDCCBWgCAQIxUqJQAgEEMCMEEKBBI2KxDUPS5TCo
RCEDJo4YDzIwMTkwOTEyMTIwMDAwWjAMBgoqgxqMmkQHAQEBBBipFE2DxCLAx2Og
E53Jt21V8kAoscU7K3wwggUNBgkqhkiG9w0BBwEwHAYIKoMajJpEAQQEEEJPR1VT
SVZfQk9HVVNJViGAggTgc8exehjJD/gtEOIrg6tK5Emaa4PJ7l8f+EtyDD/ffQay
XVAGz2MXUIQMEzmSLrnsr9NEyXvxGpvcsi7mV8tDxZU0YuyhA/C/HMh7EaBKG1hj
C7xNw+IRIUxrbRJakMQbzMWWYJupC5zRu4/Ge9i+JVOGgES2E0L5LZSZ53wmnHA0
ols1PHl3F3Z2QM3CkewqA3NP1waXQ0XXb0Oyl6Gq12B7ksm7euPWA3KctEjfYBD6
nBT6wQd57rAMeFTk5aceWd2Sb/0xMpjfCg6GzX8pAWVEU8LqTvVmlSWdx3f3fAtU
giZ+gx7jNY8A6duln8zvMQn3mtPDCa50GzSrAx8JreHRWSDr3Dp8EfJzUgfy7dWl
I9xs5bh1TMkEMk+AHWQ5sBXTZkDgVAS5m1mIbXe7dzuxKsfGxjWu1eyy9J77mtOG
o9aAOqYfxv/I8YQcgWHTeQcIO39Rmt2QsI7trRaEJ1jgj2E1To5gRCbIQWzQuyoS
6affgu/9dwPXCAt0+0XrnO5vhaKX/RWm7ve8hYsiT0vI0hdBJ3rDRkdS9VL6NlnX
OuohAqEq8b3s2koBigdri052hceAElTHD+4A4qRDiMLlFLlQqoJlpBwCtEPZsIQS
y62K7J/Towxxab5FoFjUTC5f79xPQPoKxYdgUB5AeAu5HgdWTn49Uqg4v/spTPSN
RTmDMVVyZ9qhzJfkDpH3TKCAE5t59w4gSPe/7l+MeSml9O+L9HTd9Vng3LBbIds3
uQ4cfLyyQmly81qpJjR1+Rvwo46hOm0kf2sIFi0WULmP/XzLw6b1SbiHf/jqFg7T
FTyLMkPMPMmc7/kpLmYbKyTB4ineasTUL+bDrwu+uSzFAjTcI+1sz4Wo4p7RVywB
DKSI5Ocbd3iMt4XWJWtz0KBX6nBzlV+BBTCwaGMAU4IpPBYOuvcl7TJWx/ODBjbO
4zm4T/66w5IG3tKpsVMs4Jtrh8mtVXCLTBmKDzyjBVN2X8ALGXarItRgLa7k80lJ
jqTHwKCjiAMmT/eh67KzwmqBq5+8rJuXkax0NoXcDu6xkCMNHUQBYdnskaJqC2pu
8hIsPTOrh7ieYSEuchFvu7lI0E+p7ypW65CMiy+Y/Rm5OWeHzjKkU5AbPtx/Me2v
pQRCgaPwciZunx2Ivi1+WYUBU1pGNDO7Xz7a8UHbDURkh7b+40uz2d7YQjKgrZBv
6YwLAmw1LTE4bT9PM9n7LROnX8u6ksei8yiw8gZeVu+plWHbF+0O9siKAgxZlBna
0XFgPpdzjMDTS/sfTIYXWlFj7camhsmTDRjo5G2B212evaKmKgh5ALLSFSk86ZN5
KvQvcfsp81jvJCBmDStrsUgSMzy0Og2quHOd61hRTVlYzwvJvfMzHGKdIWwYUbHZ
OKo/KLEk3E36U9PkPoZGEL2ZeCH4F9Wh3mgg0knBfEmlPnGexmBby6NXGK7VW3l6
xcJlpdMaXKNVMfl2YK8k/34Hyft06KaYLEJsxAqk1pmLEmGhdZC1OAqovVB/1agS
zpMMaB9OWWqNsTjDc7tkDt8BZ72NsAbCI9XmsX81W+NqPb6Ju1dtI09bn113LX/Z
bOSdVicQcXSpl0FnTZaHgHJdQLcU28O7yFFOblqrvcMKpctdTA1TwG9LXEFttGrl
pgjZF3edo0Cez10epK+S
"""

    def setUp(self):
        self.asn1Spec = rfc5652.ContentInfo()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.env_data_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertEqual(rfc5652.id_envelopedData, asn1Object['contentType'])

        ed, rest = der_decoder(asn1Object['content'], rfc5652.EnvelopedData())
        self.assertFalse(rest)
        self.assertTrue(ed.prettyPrint())
        self.assertEqual(asn1Object['content'], der_encoder(ed))

        kwa = ed['recipientInfos'][0]['kekri']['keyEncryptionAlgorithm']
        self.assertEqual(rfc4010.id_npki_app_cmsSeed_wrap, kwa['algorithm'])

        cea = ed['encryptedContentInfo']['contentEncryptionAlgorithm']
        self.assertEqual(rfc4010.id_seedCBC, cea['algorithm'])
        param, rest = der_decoder(
            cea['parameters'], asn1Spec=rfc4010.SeedCBCParameter())
        self.assertFalse(rest)
        self.assertTrue(param.prettyPrint())
        self.assertEqual(cea['parameters'], der_encoder(param))

        iv = univ.OctetString(hexValue='424f47555349565f424f475553495621')
        self.assertEqual(iv, param)

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.env_data_pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))
        self.assertIn(asn1Object['contentType'], rfc5652.cmsContentTypesMap.keys())

        kekri = asn1Object['content']['recipientInfos'][0]['kekri']
        kwa = kekri['keyEncryptionAlgorithm']
        self.assertEqual(rfc4010.id_npki_app_cmsSeed_wrap, kwa['algorithm'])

        eci = asn1Object['content']['encryptedContentInfo']
        cea = eci['contentEncryptionAlgorithm']
        self.assertEqual(rfc4010.id_seedCBC, cea['algorithm'])

        iv = univ.OctetString(hexValue='424f47555349565f424f475553495621')
        self.assertEqual(iv, cea['parameters'])

class SMIMECapabilitiesTestCase(unittest.TestCase):
    smime_capabilities_pem_text = "MB4wDAYIKoMajJpEAQQFADAOBgoqgxqMmkQHAQEBBQA="

    def setUp(self):
        self.asn1Spec = rfc5751.SMIMECapabilities()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.smime_capabilities_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        alg_oid_list = [ ]
        for cap in asn1Object:
            self.assertTrue(cap['parameters'].hasValue())
            self.assertEqual(cap['parameters'], der_encoder(rfc4010.SeedSMimeCapability("")))
            alg_oid_list.append(cap['capabilityID'])

        self.assertIn(rfc4010.id_seedCBC, alg_oid_list)
        self.assertIn(rfc4010.id_npki_app_cmsSeed_wrap, alg_oid_list)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())




#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc4211
from pyasn1_alt_modules import rfc4212


class OpenPGPCertTemplateTestCase(unittest.TestCase):
    pem_text = """\
MIIEiwIBMDAAMIIEgjCCBH4GCSsGAQUFBwUBBzCCBG8GCisGAQUFBwUBBwIwggRf
BIIEW5kBogQ8WCeiEQDj+53/////////////////////////////////////////
////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////8r7wCg
/37///////////////////+6cQP/aLz/////////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
//9WcQP+OB//////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
///////////////////////////////////////////////yY7QZQWz/////////
/////////////////20+iQBJBBARAgAJBQI8WCeiAhsDAAoJEENc////BndawgCg
6wD///////////////////8bdQCg9OT///////////////////+oPbkCDQQ8WCei
EAgA9kL/////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
/////////////////////ws7AAICB/43uv//////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////3yGJAEkEGBECAAkF
AjxYJ6ICGwwACgkQQ1z///8Gd8feAJ4hM////////////////////zkbAJ9k1///
/////////////////2MI
"""

    def setUp(self):
        self.asn1Spec = rfc4211.CertRequest()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        self.assertEqual(48, asn1Object['certReqId'])

        found = False

        for ctrl in asn1Object['controls']:
            if ctrl['type'] == rfc4212.id_regCtrl_altCertTemplate:
                act, rest = der_decoder(ctrl['value'],
                    asn1Spec=rfc4212.AttributeTypeAndValue())
                self.assertFalse(rest)
                self.assertTrue(act.prettyPrint())
                self.assertEqual(ctrl['value'], der_encoder(act))

                self.assertEqual(rfc4212.id_openPGPCertTemplateExt, act['type'])
                opgpcte, rest = der_decoder(act['value'],
                    asn1Spec=rfc4212.OpenPGPCertTemplateExtended())
                self.assertFalse(rest)
                self.assertTrue(opgpcte.prettyPrint())
                self.assertEqual(act['value'], der_encoder(opgpcte))
                
                self.assertTrue(opgpcte.hasValue())
                found = True

        self.assertTrue(found)

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate,
            asn1Spec=self.asn1Spec, decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        self.assertEqual(48, asn1Object['certReqId'])
        
        found = False

        for ctrl in asn1Object['controls']:
            if ctrl['type'] == rfc4212.id_regCtrl_altCertTemplate:
                self.assertEqual(rfc4212.id_openPGPCertTemplateExt,
                    ctrl['value']['type'])
                self.assertTrue(ctrl['value']['value'].hasValue())
                found = True

        self.assertTrue(found)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-alt-modules software.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc8398
from pyasn1_alt_modules import opentypemap


class EAITestCase(unittest.TestCase):
    pem_text = "oCAGCCsGAQUFBwgJoBQMEuiAgeW4q0BleGFtcGxlLmNvbQ=="

    def setUp(self):
        self.asn1Spec = rfc5280.GeneralName()

    def testDerCodec(self):
        otherNamesMap = opentypemap.get('otherNamesMap')

        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        self.assertIn(asn1Object['otherName']['type-id'], otherNamesMap)
        self.assertEqual(rfc8398.id_on_SmtpUTF8Mailbox,
                         asn1Object['otherName']['type-id'])

        eai, rest = der_decoder(asn1Object['otherName']['value'],
            asn1Spec=otherNamesMap[asn1Object['otherName']['type-id']])
        self.assertFalse(rest)
        self.assertTrue(eai.prettyPrint())
        self.assertEqual(asn1Object['otherName']['value'], der_encoder(eai))

        self.assertEqual(u'\u8001', eai[0])
        self.assertEqual(u'\u5E2B', eai[1])

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate,
            asn1Spec=self.asn1Spec, decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        self.assertEqual(rfc8398.id_on_SmtpUTF8Mailbox,
                         asn1Object['otherName']['type-id'])
        self.assertEqual(u'\u8001', asn1Object['otherName']['value'][0])
        self.assertEqual(u'\u5E2B', asn1Object['otherName']['value'][1])


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2020-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Identifiers for HMAC-SHA-224, HMAC-SHA-256, HMAC-SHA-384,
#   and HMAC-SHA-512
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc4231.txt
# https://www.rfc-editor.org/rfc/rfc8018.txt
#

from pyasn1_alt_modules import rfc8018


# The HMAC object identifiers are already defined in RFC 8018

id_hmacWithSHA224 = rfc8018.id_hmacWithSHA224

id_hmacWithSHA256 = rfc8018.id_hmacWithSHA256

id_hmacWithSHA384 = rfc8018.id_hmacWithSHA384

id_hmacWithSHA512 = rfc8018.id_hmacWithSHA512


# The Algorithm Identifiers Map is updated by importing rfc8018.
# To save looking it up, the map is updated with these entries:
#  _algorithmIdentifierMapUpdate = {
#     id_hmacWithSHA224: univ.Null(),
#     id_hmacWithSHA256: univ.Null(),
#     id_hmacWithSHA384: univ.Null(),
#     id_hmacWithSHA512: univ.Null(),
#  }



# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
#
# Copyright (c) 2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# ML-DSA Signature Algorithm for the CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9882.txt

from pyasn1_alt_modules import rfc9881


# Object Identifiers

nistAlgorithms = rfc9881.nistAlgorithms

sigAlgs = rfc9881.sigAlgs

id_ml_dsa_44 = rfc9881.id_ml_dsa_44

id_ml_dsa_65 = rfc9881.id_ml_dsa_65

id_ml_dsa_87 = rfc9881.id_ml_dsa_87


# Signature Algorithm Identifier

sa_ml_dsa_44 = rfc9881.sa_ml_dsa_44

sa_ml_dsa_65 = rfc9881.sa_ml_dsa_65

sa_ml_dsa_87 = rfc9881.sa_ml_dsa_87


# No need to add the id_ml_dsa_* object identifiers to the
# algorithmIdentifierMap or the smimeCapabilityMap because the
# parameters are always absent.



#
# This file is part of pyasn1-alt-modules software.
#
# This very simple manager for opentype maps allows various related
# ASN.1 modules to share the same maps.
#
# Created by Russ Housley
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#

from collections import defaultdict

map_of_opentype_maps = defaultdict(dict)


def get (map_name):
    """Get the named opentype map, creating an empty one if needed."""
    return map_of_opentype_maps[map_name]



# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
#
# Copyright (c) 2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# HSS/LMS, XMSS, and XMSS^MT Hash-based Signature Algorithms for X.509
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8708.txt
# https://www.rfc-editor.org/rfc/rfc9802.txt


from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc8708


# HSS/LMS Public Key and Signature Algorithm Identifier

id_alg_hss_lms_hashsig = rfc8708.id_alg_hss_lms_hashsig


class HSS_LMS_HashSig_Signature(univ.OctetString):
    pass


HSS_LMS_HashSig_PublicKey = rfc8708.HSS_LMS_HashSig_PublicKey


sa_HSS_LMS_HashSig = rfc5280.AlgorithmIdentifier()
sa_HSS_LMS_HashSig['algorithm'] = id_alg_hss_lms_hashsig
# sa_HSS_LMS_HashSig['parameters'] is alway absent


pk_HSS_LMS_HashSig = rfc5280.SubjectPublicKeyInfo()
pk_HSS_LMS_HashSig['algorithm'] = sa_HSS_LMS_HashSig
# pk_HSS_LMS_HashSig['subjectPublicKey'] CONTAINS the
#     HSS/LMS public key without any ASN.1 encoding



# XMSS Public Key and Signature Algorithm Identifier

id_alg_xmss_hashsig = univ.ObjectIdentifier('1.3.6.1.5.5.7.6.34')


class XMSS_HashSig_Signature(univ.OctetString):
    pass


class XMSS_HashSig_PublicKey(univ.OctetString):
    pass


sa_XMSS_HashSig = rfc5280.AlgorithmIdentifier()
sa_XMSS_HashSig['algorithm'] = id_alg_xmss_hashsig
# sa_XMSS_HashSig['parameters'] is alway absent


pk_XMSS_HashSig = rfc5280.SubjectPublicKeyInfo()
pk_XMSS_HashSig['algorithm'] = sa_XMSS_HashSig
# pk_XMSS_HashSig['subjectPublicKey'] CONTAINS the
#     XMSS public key without any ASN.1 encoding


# XMSS^MT Public Key and Signature Algorithm Identifier

id_alg_xmssmt_hashsig = univ.ObjectIdentifier('1.3.6.1.5.5.7.6.35')


class XMSSMT_HashSig_Signature(univ.OctetString):
    pass


class XMSSMT_HashSig_PublicKey(univ.OctetString):
    pass


sa_XMSSMT_HashSig = rfc5280.AlgorithmIdentifier()
sa_XMSSMT_HashSig['algorithm'] = id_alg_xmssmt_hashsig
# sa_XMSSMT_HashSig['parameters'] is alway absent


pk_XMSSMT_HashSig = rfc5280.SubjectPublicKeyInfo()
pk_XMSSMT_HashSig['algorithm'] = sa_XMSSMT_HashSig
# pk_XMSSMT_HashSig['subjectPublicKey'] CONTAINS the
#     XMSS^MT public key without any ASN.1 encoding



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2022-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder
from pyasn1.type import univ
from pyasn1.type import error

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import rfc9323

class BadFileNameTestCase(unittest.TestCase):
    def testRaisesValueConstraintError(self):
        self.assertRaises(
            error.ValueConstraintError,
            rfc9323.PortableFilename, 'abc+'
        )

class RPKISignedChecklistTestCase(unittest.TestCase):
    pem_text = """\
MIIGjwYJKoZIhvcNAQcCoIIGgDCCBnwCAQMxDTALBglghkgBZQMEAgEwgZYGCyqG
SIb3DQEJEAEwoIGGBIGDMIGAMBWhEzARMA8EAgACMAkDBwAgAQZ8IIwwCwYJYIZI
AWUDBAIBMFowNBYQYjQyX2lwdjZfbG9hLnBuZwQglRbdZL58FyW5/KEXEg5Y6NhC
pSBoczmbPd/8kcS2rPAwIgQgCuE5RyIAXNkvTGqgJNXWs+LmfWKfEXINlHimM6EX
ocegggQfMIIEGzCCAwOgAwIBAgIBATANBgkqhkiG9w0BAQsFADAzMTEwLwYDVQQD
EygzOGUxNGY5MmZkYzdjY2ZiZmMxODIzNjE1MjNhZTI3ZDY5N2U5NTJmMB4XDTIy
MDUyNzE5NDUwMloXDTIzMDUyNzE5NDUwMlowDTELMAkGA1UEAwwCRUUwggEiMA0G
CSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDk9EeZ+bni2y4wJlIbmP2YH+uVw5OP
ewTHQn93Zi4d6N+BUQxGYzJhGI7ZsarnK2iuYxs6E+bUtPXPYJzDx8NzGhfzOITt
GoJlim5isKIBBlPi37IY75swenSQOoPeY6VOLl3elNuUoIdL+S1jI9BksQSXX6LG
6YJCxxTDGg/27F+dDN9DKCAVB8ZxXG9hnOTzv8BCD/D0YU/irAuIZVj5xH3xjM/J
gOqzmxygsuJUGIcy5JcUhhYVQ5Kqx2FBpaaSklo/Rg9sDzbMsOIhwb6Ylk6W4d1j
3zMyORTdg9eL1KXBN8yfstjC0s81y3nbSgWPiWuprHnXPI+bmZi2j+7jAgMBAAGj
ggFeMIIBWjAOBgNVHQ8BAf8EBAMCB4AwHQYDVR0OBBYEFKDCf75nJYStTKGtU/BK
BYMEgonnMB8GA1UdIwQYMBaAFDjhT5L9x8z7/BgjYVI64n1pfpUvMBgGA1UdIAEB
/wQOMAwwCgYIKwYBBQUHDgIwIgYIKwYBBQUHAQcBAf8EEzARMA8EAgACMAkDBwAg
AQZ8IIwwZAYIKwYBBQUHAQEEWDBWMFQGCCsGAQUFBzAChkhyc3luYzovL3Jwa2ku
cmlwZS5uZXQvcmVwb3NpdG9yeS9ERUZBVUxUL09PRlBrdjNIelB2OEdDTmhVanJp
ZldsLWxTOC5jZXIwZAYDVR0fBF0wWzBZoFegVYZTcnN5bmM6Ly9jaGxvZS5zb2Jv
cm5vc3QubmV0L3Jwa2kvUklQRS1ubGpvYnNuaWpkZXJzL09PRlBrdjNIelB2OEdD
TmhVanJpZldsLWxTOC5jcmwwDQYJKoZIhvcNAQELBQADggEBAJoDJvJxThQol9gB
n3MswmFk3LigqG3Sv2HeKRmyeEq8hPHudPo0MbHAbgps+lQPUi8RQaKyY69qW1lR
Mqej5eDOV164Yo7nRL9tSuBC1ne0kegApYNcq8nRZbH76Kb3TxgXVsG/D9o11AJF
6M32lSNlO+tbC7MMDhO0xNXjLpjXDmOpohg0wy3DZ+nbtIlK4UuWgKunz/iv36X6
uFakeVhtQLnFTncXdTvhRD+r8NwQqY34yW+q2ZFI4paNQPC8emXWIpqgVorISYmw
AusVcCpICedJrUsY+Drcc3o9Ib4zyvN4ldWe/T/oUtjA5ZIZoejjIIiBSVbnaGmz
eSkT31wxggGqMIIBpgIBA4AUoMJ/vmclhK1Moa1T8EoFgwSCiecwCwYJYIZIAWUD
BAIBoGswGgYJKoZIhvcNAQkDMQ0GCyqGSIb3DQEJEAEwMBwGCSqGSIb3DQEJBTEP
Fw0yMjA1MjcxOTQ1MzRaMC8GCSqGSIb3DQEJBDEiBCBErCuXeuq17Q+2ZRHu0qxP
onZIDG1nQIenPR762BFoDTANBgkqhkiG9w0BAQEFAASCAQDa9VOFllioS7jYxDxW
BbmTeVqWptuU71lBXD/Kggzb3JHhrqHFmzPYIJbJqKo5vu6g3BjT93xGO6zqB2eN
nXPsNKU7krThjCkjhg/8vmzEBEG7+iFdKcY5XrDqhhGY9y8TGQJgq00bCBAwbmMS
giDYfsT7hev6SUAvoC+x8fmT40rTMpc8bkR5mITpqaaRigxhUij2W/aYS+u9/cwH
l/Q7/nCdUBJe+x2ubsbHcGRSQ7EO1cXT1jiToYlfV+VHRUFtl+ud6nVgJ/Nln9Ob
pLpXzrBiy+/+b5KX87CXkMsjHKRYW92mMnXkC1GKj55YL8Eb6Zi3o81X5fHBmhqv
lj/4
"""

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)
        
        layers = { }
        layers.update(rfc5652.cmsContentTypesMap)
        self.assertIn(rfc9323.id_ct_signedChecklist, layers)

        getNextLayer = {
            rfc5652.id_ct_contentInfo: lambda x: x['contentType'],
            rfc5652.id_signedData: lambda x: x['encapContentInfo']['eContentType'],
        }

        getNextSubstrate = {
            rfc5652.id_ct_contentInfo: lambda x: x['content'],
            rfc5652.id_signedData: lambda x: x['encapContentInfo']['eContent'],
        }

        layer = rfc5652.id_ct_contentInfo
        while layer in getNextLayer:
            asn1Object, rest = der_decoder(substrate, asn1Spec=layers[layer])
            self.assertFalse(rest)
            self.assertTrue(asn1Object.prettyPrint())
            self.assertEqual(substrate, der_encoder(asn1Object))

            substrate = getNextSubstrate[layer](asn1Object)
            layer = getNextLayer[layer](asn1Object)

        self.assertEqual(rfc9323.id_ct_signedChecklist, layer)

        asn1Object, rest = der_decoder(substrate,
            asn1Spec=rfc9323.RpkiSignedChecklist())
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        count = 0
        filenames = [ ]
        for fnah in asn1Object['checkList']:
            count += 1
            if fnah['fileName'].hasValue():
                filenames.append(fnah['fileName'])

        self.assertEqual(2, count)
        self.assertIn('b42_ipv6_loa.png', filenames)

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pem_text)

        asn1Object, rest = der_decoder(substrate,
            asn1Spec=rfc5652.ContentInfo(), decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        substrate = asn1Object['content']['encapContentInfo']['eContent']
        asn1Object, rest = der_decoder(substrate,
            asn1Spec=rfc9323.RpkiSignedChecklist(), decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        count = 0
        filenames = [ ]
        for fnah in asn1Object['checkList']:
            count += 1
            if fnah['fileName'].hasValue():
                filenames.append(fnah['fileName'])

        self.assertEqual(2, count)
        self.assertIn('b42_ipv6_loa.png', filenames)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2024-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc9509


class NFVEKUTestCase(unittest.TestCase):
    cert_pem_text = """\
MIIDWzCCAuKgAwIBAgIJAKWzVCgbsG5TMAoGCCqGSM49BAMDMD8xCzAJBgNVBAYT
AlVTMQswCQYDVQQIDAJWQTEQMA4GA1UEBwwHSGVybmRvbjERMA8GA1UECgwIQm9n
dXMgQ0EwHhcNMjMwOTI2MTcwNDE0WhcNMjQwOTI1MTcwNDE0WjBaMQswCQYDVQQG
EwJVUzEqMCgGA1UECgwhNWdjLm1uYzAxMi5tY2MzNDUuM2dwcG5ldHdvcmsub3Jn
MR8wHQYDVQQDDBZhbWYxLmNsdXN0ZXIxLm5ldDIuYW1mMHYwEAYHKoZIzj0CAQYF
K4EEACIDYgAE7RSQ7GmVAzQsfifYsF9C45i1f86fgmfFl6ELlUgYjt5fG2ZWwIE3
oF/SQAmAD5cvusDYqIpx4iGOSCJnvgId1iMvphRTqw4gFK6Q49Zk/thG2BMdImny
4XjrwnRWzfGwo4IBjTCCAYkwgbcGA1UdEQSBrzCBrII4YW1mMS5jbHVzdGVyMS5u
ZXQyLmFtZi41Z2MubW5jMDEyLm1jYzM0NS4zZ3BwbmV0d29yay5vcmeCQWFtZjEu
Y2FsbGJhY2suY2x1c3RlcjEubmV0Mi5hbWYuNWdjLm1uYzAxMi5tY2MzNDUuM2dw
cG5ldHdvcmsub3Jnhi11cm46dXVpZDpmODFkNGZhZS03ZGVjLTExZDAtYTc2NS0w
MGEwYzkxZTZiZjYwCwYDVR0PBAQDAgeAMCcGA1UdJQQgMB4GCCsGAQUFBwMBBggr
BgEFBQcDAgYIKwYBBQUHAycwHQYDVR0OBBYEFCIMH3AXpn6uXdyu6WrlwdCAbwyy
MB8GA1UdIwQYMBaAFPI12zQE2qVV8r1pA5mwYuziFQjBMBMGCCsGAQUFBwEiBAcw
BRYDQU1GMEIGCWCGSAGG+EIBDQQ1FjNUaGlzIGNlcnRpZmljYXRlIGNhbm5vdCBi
ZSB0cnVzdGVkIGZvciBhbnkgcHVycG9zZS4wCgYIKoZIzj0EAwMDZwAwZAIwHYnv
q2hGDO6v+keLrOU76L5NacqSW8cqtumzjMKZrUZWDyyCWw1w3a2Bb/2A3StDAjAK
bcZvtuatkNjj3fCInbwVPeWHzAGgf/P4yAoKCsQHjOo+moTnE7hUKYp3Zw6KvPs=
"""

    def setUp(self):
        self.asn1Spec = rfc5280.Certificate()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.cert_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        found = False
        for extn in asn1Object['tbsCertificate']['extensions']:
            if extn['extnID'] in rfc5280.certificateExtensionsMap:
                extnValue, rest = der_decoder(
                    extn['extnValue'],
                    asn1Spec=rfc5280.certificateExtensionsMap[extn['extnID']])
                self.assertFalse(rest)
                self.assertTrue(extnValue.prettyPrint())
                self.assertEqual(extn['extnValue'], der_encoder(extnValue))

                if extn['extnID'] == rfc5280.id_ce_extKeyUsage:
                    self.assertIn(rfc9509.id_kp_oauthAccessTokenSigning, extnValue)
                    found = True

        self.assertTrue(found)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Extensible Messaging and Presence Protocol (XMPP)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc6120.txt
#

from pyasn1.type import char
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import opentypemap

otherNamesMap = opentypemap.get('otherNamesMap')

MAX = float('inf')


# XmppAddr Identifier Type as specified in Section 13.7.1.4. of RFC 6120

id_pkix = rfc5280.id_pkix

id_on = id_pkix + (8, )

id_on_xmppAddr = id_on + (5, )


class XmppAddr(char.UTF8String):
    pass


# Update the Other Names Map

_otherNamesMapUpdate = {
    id_on_xmppAddr: XmppAddr(),
}

otherNamesMap.update(_otherNamesMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# RPKI Route Origin Authorizations (ROAs)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc6482.txt
# https://www.rfc-editor.org/errata/eid5881
#

from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import tag
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import opentypemap

cmsContentTypesMap = opentypemap.get('cmsContentTypesMap')

MAX = float('inf')


id_ct_routeOriginAuthz = univ.ObjectIdentifier('1.2.840.113549.1.9.16.1.24')


class ASID(univ.Integer):
    pass


class IPAddress(univ.BitString):
    pass


class ROAIPAddress(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('address', IPAddress()),
        namedtype.OptionalNamedType('maxLength', univ.Integer())
    )


class ROAIPAddressFamily(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('addressFamily',
            univ.OctetString().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(2, 3))),
        namedtype.NamedType('addresses',
            univ.SequenceOf(componentType=ROAIPAddress()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)))
    )


class RouteOriginAttestation(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType('version',
            univ.Integer().subtype(explicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 0)).subtype(value=0)),
        namedtype.NamedType('asID', ASID()),
        namedtype.NamedType('ipAddrBlocks',
            univ.SequenceOf(componentType=ROAIPAddressFamily()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)))
    )


# Update the CMS Content Types Map

_cmsContentTypesMapUpdate = {
    id_ct_routeOriginAuthz: RouteOriginAttestation(),
}

cmsContentTypesMap.update(_cmsContentTypesMapUpdate)



# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
#
# Copyright (c) 2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# SLH-DSA Signature Algorithm for X.509 Certificates
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9909.txt

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc9814


# Import definitions for pure mode of SLH-DSA from rfc9814

nistAlgorithms = rfc9814.nistAlgorithms

sigAlgs = rfc9814.sigAlgs

id_slh_dsa_sha2_128s = rfc9814.id_slh_dsa_sha2_128s

id_slh_dsa_sha2_128f = rfc9814.id_slh_dsa_sha2_128f

id_slh_dsa_sha2_192s = rfc9814.id_slh_dsa_sha2_192s

id_slh_dsa_sha2_192f = rfc9814.id_slh_dsa_sha2_192f

id_slh_dsa_sha2_256s = rfc9814.id_slh_dsa_sha2_256s

id_slh_dsa_sha2_256f = rfc9814.id_slh_dsa_sha2_256f

id_slh_dsa_shake_128s = rfc9814.id_slh_dsa_shake_128s

id_slh_dsa_shake_128f = rfc9814.id_slh_dsa_shake_128f

id_slh_dsa_shake_192s = rfc9814.id_slh_dsa_shake_192s

id_slh_dsa_shake_192f = rfc9814.id_slh_dsa_shake_192f

id_slh_dsa_shake_256s = rfc9814.id_slh_dsa_shake_256s

id_slh_dsa_shake_256s = rfc9814.id_slh_dsa_shake_256s

sa_slh_dsa_sha2_128s = rfc9814.sa_slh_dsa_sha2_128s

sa_slh_dsa_sha2_128f = rfc9814.sa_slh_dsa_sha2_128f

sa_slh_dsa_sha2_192s = rfc9814.sa_slh_dsa_sha2_192s

sa_slh_dsa_sha2_192f = rfc9814.sa_slh_dsa_sha2_192f

sa_slh_dsa_sha2_256s = rfc9814.sa_slh_dsa_sha2_256s

sa_slh_dsa_sha2_256f = rfc9814.sa_slh_dsa_sha2_256f

sa_slh_dsa_shake_128s = rfc9814.sa_slh_dsa_shake_128s

sa_slh_dsa_shake_128f = rfc9814.sa_slh_dsa_shake_128f

sa_slh_dsa_shake_192s = rfc9814.sa_slh_dsa_shake_192s

sa_slh_dsa_shake_192f = rfc9814.sa_slh_dsa_shake_192f

sa_slh_dsa_shake_256s = rfc9814.sa_slh_dsa_shake_256s

sa_slh_dsa_shake_256f = rfc9814.sa_slh_dsa_shake_256f

pk_slh_dsa_sha2_128s = rfc9814.pk_slh_dsa_sha2_128s

pk_slh_dsa_sha2_128f = rfc9814.pk_slh_dsa_sha2_128f

pk_slh_dsa_sha2_192s = rfc9814.pk_slh_dsa_sha2_192s

pk_slh_dsa_sha2_192f = rfc9814.pk_slh_dsa_sha2_192f

pk_slh_dsa_sha2_256s = rfc9814.pk_slh_dsa_sha2_256s

pk_slh_dsa_sha2_256f = rfc9814.pk_slh_dsa_sha2_256f

pk_slh_dsa_shake_128s = rfc9814.pk_slh_dsa_shake_128s

pk_slh_dsa_shake_128f = rfc9814.pk_slh_dsa_shake_128f

pk_slh_dsa_shake_192s = rfc9814.pk_slh_dsa_shake_192s

pk_slh_dsa_shake_192f = rfc9814.pk_slh_dsa_shake_192f

pk_slh_dsa_shake_256s = rfc9814.pk_slh_dsa_shake_256s

pk_slh_dsa_shake_256f = rfc9814.pk_slh_dsa_shake_256f

SLH_DSA_PrivateKey = rfc9814.SLH_DSA_PrivateKey

SLH_DSA_PublicKey = rfc9814.SLH_DSA_PublicKey


# Object Identifier definitions for pre-hash mode of SLH-DSA

id_hash_slh_dsa_sha2_128s_with_sha256 = sigAlgs + (35, )	

id_hash_slh_dsa_sha2_128f_with_sha256 = sigAlgs + (36, )	
		 		
id_hash_slh_dsa_sha2_192s_with_sha512 = sigAlgs + (37, )	
		 		
id_hash_slh_dsa_sha2_192f_with_sha512 = sigAlgs + (38, )	
		 		
id_hash_slh_dsa_sha2_256s_with_sha512 = sigAlgs + (39, )	
		 		
id_hash_slh_dsa_sha2_256f_with_sha512 = sigAlgs + (40, )	
		 		
id_hash_slh_dsa_shake_128s_with_shake128 = sigAlgs + (41, )	
		 		
id_hash_slh_dsa_shake_128f_with_shake128 = sigAlgs + (42, )	
		 		
id_hash_slh_dsa_shake_192s_with_shake256 = sigAlgs + (43, )	
		 		
id_hash_slh_dsa_shake_192f_with_shake256 = sigAlgs + (44, )	
		 		
id_hash_slh_dsa_shake_256s_with_shake256 = sigAlgs + (45, )	

id_hash_slh_dsa_shake_256f_with_shake256 = sigAlgs + (46, )	


# Signature algorithm definitions for pre-hash mode of SLH-DSA

sa_hash_slh_dsa_sha2_128s_with_sha256 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_sha2_128s_with_sha256['algorithm'] = id_hash_slh_dsa_sha2_128s_with_sha256	
# sa_hash_slh_dsa_sha2_128s_with_sha256['parameters'] is always absent

sa_hash_slh_dsa_sha2_128f_with_sha256 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_sha2_128f_with_sha256['algorithm'] = id_hash_slh_dsa_sha2_128f_with_sha256	
# sa_hash_slh_dsa_sha2_128f_with_sha256['parameters'] is always absent

sa_hash_slh_dsa_sha2_192s_with_sha512 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_sha2_192s_with_sha512['algorithm'] = id_hash_slh_dsa_sha2_192s_with_sha512	
# sa_hash_slh_dsa_sha2_192s_with_sha512['parameters'] is always absent

sa_hash_slh_dsa_sha2_192f_with_sha512 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_sha2_192f_with_sha512['algorithm'] = id_hash_slh_dsa_sha2_192f_with_sha512	
# sa_hash_slh_dsa_sha2_192f_with_sha512['parameters'] is always absent

sa_hash_slh_dsa_sha2_256s_with_sha512 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_sha2_256s_with_sha512['algorithm'] = id_hash_slh_dsa_sha2_256s_with_sha512	
# sa_hash_slh_dsa_sha2_256s_with_sha512['parameters'] is always absent

sa_hash_slh_dsa_sha2_256f_with_sha512 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_sha2_256f_with_sha512['algorithm'] = id_hash_slh_dsa_sha2_256f_with_sha512	
# sa_hash_slh_dsa_sha2_256f_with_sha512['parameters'] is always absent

sa_hash_slh_dsa_shake_128s_with_shake128 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_shake_128s_with_shake128['algorithm'] = id_hash_slh_dsa_shake_128s_with_shake128	
# sa_hash_slh_dsa_shake_128s_with_shake128['parameters'] is always absent

sa_hash_slh_dsa_shake_128f_with_shake128 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_shake_128f_with_shake128['algorithm'] = id_hash_slh_dsa_shake_128f_with_shake128	
# sa_hash_slh_dsa_shake_128f_with_shake128['parameters'] is always absent

sa_hash_slh_dsa_shake_192s_with_shake256 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_shake_192s_with_shake256['algorithm'] = id_hash_slh_dsa_shake_192s_with_shake256	
# sa_hash_slh_dsa_shake_192s_with_shake256['parameters'] is always absent

sa_hash_slh_dsa_shake_192f_with_shake256 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_shake_192f_with_shake256['algorithm'] = id_hash_slh_dsa_shake_192f_with_shake256	
# sa_hash_slh_dsa_shake_192f_with_shake256['parameters'] is always absent

sa_hash_slh_dsa_shake_256s_with_shake256 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_shake_256s_with_shake256['algorithm'] = id_hash_slh_dsa_shake_256s_with_shake256	
# sa_hash_slh_dsa_shake_256s_with_shake256['parameters'] is always absent

sa_hash_slh_dsa_shake_256f_with_shake256 = rfc5280.AlgorithmIdentifier()
sa_hash_slh_dsa_shake_256f_with_shake256['algorithm'] = id_hash_slh_dsa_shake_256f_with_shake256
# sa_hash_slh_dsa_shake_256f_with_shake256['parameters'] is always absent


# Public key definitions for pre-hash mode of SLH-DSA

pk_hash_slh_dsa_sha2_128s_with_sha256 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_sha2_128s_with_sha256['algorithm'] = sa_hash_slh_dsa_sha2_128s_with_sha256
# pk_hash_slh_dsa_sha2_128s_with_sha256['subjectPublicKey'] is the raw public key

pk_hash_slh_dsa_sha2_128f_with_sha256 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_sha2_128f_with_sha256['algorithm'] = sa_hash_slh_dsa_sha2_128f_with_sha256
# pk_hash_slh_dsa_sha2_128f_with_sha256['subjectPublicKey'] is the raw public key

pk_hash_slh_dsa_sha2_192s_with_sha512 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_sha2_192s_with_sha512['algorithm'] = sa_hash_slh_dsa_sha2_192s_with_sha512
# pk_hash_slh_dsa_sha2_192s_with_sha512['subjectPublicKey'] is the raw public key

pk_hash_slh_dsa_sha2_192f_with_sha512 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_sha2_192f_with_sha512['algorithm'] = sa_hash_slh_dsa_sha2_192f_with_sha512
# pk_hash_slh_dsa_sha2_192f_with_sha512['subjectPublicKey'] is the raw public key

pk_hash_slh_dsa_sha2_256s_with_sha512 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_sha2_256s_with_sha512['algorithm'] = sa_hash_slh_dsa_sha2_256s_with_sha512
# pk_hash_slh_dsa_sha2_256s_with_sha512['subjectPublicKey'] is the raw public key

pk_hash_slh_dsa_sha2_256f_with_sha512 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_sha2_256f_with_sha512['algorithm'] = sa_hash_slh_dsa_sha2_256f_with_sha512
# pk_hash_slh_dsa_sha2_256f_with_sha512['subjectPublicKey'] is the raw public key

pk_hash_slh_dsa_shake_128s_with_shake128 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_shake_128s_with_shake128['algorithm'] = sa_hash_slh_dsa_shake_128s_with_shake128	
# pk_hash_slh_dsa_shake_128s_with_shake128['subjectPublicKey'] is the raw public key

pk_hash_slh_dsa_shake_128f_with_shake128 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_shake_128f_with_shake128['algorithm'] = sa_hash_slh_dsa_shake_128f_with_shake128	
# pk_hash_slh_dsa_shake_128f_with_shake128['subjectPublicKey'] is the raw public key

pk_hash_slh_dsa_shake_192s_with_shake256 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_shake_192s_with_shake256['algorithm'] = sa_hash_slh_dsa_shake_192s_with_shake256
# pk_hash_slh_dsa_shake_192s_with_shake256['subjectPublicKey'] is the raw public key

pk_hash_slh_dsa_shake_192f_with_shake256 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_shake_192f_with_shake256['algorithm'] = sa_hash_slh_dsa_shake_192f_with_shake256
# pk_hash_slh_dsa_shake_192f_with_shake256['subjectPublicKey'] is the raw public key

pk_hash_slh_dsa_shake_256s_with_shake256 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_shake_256s_with_shake256['algorithm'] = sa_hash_slh_dsa_shake_256s_with_shake256
# pk_hash_slh_dsa_shake_256s_with_shake256['subjectPublicKey'] is the raw public key

pk_hash_slh_dsa_shake_256f_with_shake256 = rfc5280.SubjectPublicKeyInfo()
pk_hash_slh_dsa_shake_256f_with_shake256['algorithm'] = sa_hash_slh_dsa_shake_256f_with_shake256
# pk_hash_slh_dsa_shake_256f_with_shake256['subjectPublicKey'] is the raw public key



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# GSS-API Tokens
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc2743.txt
#

from pyasn1.type import namedtype
from pyasn1.type import namedval
from pyasn1.type import opentype
from pyasn1.type import tag
from pyasn1.type import univ

from pyasn1_alt_modules import opentypemap

gssapiMechTypeMap = opentypemap.get('gssapiMechTypeMap')


class PerMsgToken(univ.Any):
    pass


class SealedMessage(univ.Any):
    pass


class SubsequentContextToken(univ.Any):
    pass


class MechType(univ.ObjectIdentifier):
    pass


class InitialContextToken(univ.Sequence):
    pass

InitialContextToken.tagSet = univ.Sequence.tagSet.tagImplicitly(
    tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 0)
)

InitialContextToken.componentType = namedtype.NamedTypes(
    namedtype.NamedType('thisMech', MechType()),
    namedtype.NamedType('innerContextToken', univ.Any(),
        openType=opentype.OpenType('thisMech', gssapiMechTypeMap)
    )
)


# For DASS (RFC 1507), the MechType is 1.3.12.2.1011.7.5.
# For Kerberos V5 (RFC 1964), the MechType is 1.2.840.113554.1.2.2.
# If modules are ever written for these RFCs, the module should add 
# entries for these object identifiers to gssapiMechTypeMap.



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc4334
from pyasn1_alt_modules import opentypemap


class CertificateTestCase(unittest.TestCase):
    cert_pem_text = """\
MIICqzCCAjCgAwIBAgIJAKWzVCgbsG4/MAoGCCqGSM49BAMDMD8xCzAJBgNVBAYT
AlVTMQswCQYDVQQIDAJWQTEQMA4GA1UEBwwHSGVybmRvbjERMA8GA1UECgwIQm9n
dXMgQ0EwHhcNMTkwNzE5MTk0MjQ3WhcNMjAwNzE4MTk0MjQ3WjBjMQswCQYDVQQG
EwJVUzELMAkGA1UECBMCVkExEDAOBgNVBAcTB0hlcm5kb24xGzAZBgNVBAoTElZp
Z2lsIFNlY3VyaXR5IExMQzEYMBYGA1UEAxMPZWFwLmV4YW1wbGUuY29tMHYwEAYH
KoZIzj0CAQYFK4EEACIDYgAEMMbnIp2BUbuyMgH9HhNHrh7VBy7ql2lBjGRSsefR
Wa7+vCWs4uviW6On4eem5YoP9/UdO7DaIL+/J9/3DJHERI17oFxn+YWiE4JwXofy
QwfSu3cncVNMqpiDjEkUGGvBo4HTMIHQMAsGA1UdDwQEAwIHgDBCBglghkgBhvhC
AQ0ENRYzVGhpcyBjZXJ0aWZpY2F0ZSBjYW5ub3QgYmUgdHJ1c3RlZCBmb3IgYW55
IHB1cnBvc2UuMB0GA1UdDgQWBBSDjPGr7M742rsE4oQGwBvGvllZ+zAfBgNVHSME
GDAWgBTyNds0BNqlVfK9aQOZsGLs4hUIwTAeBggrBgEFBQcBDQQSMBAEB0V4YW1w
bGUEBUJvZ3VzMB0GA1UdJQQWMBQGCCsGAQUFBwMOBggrBgEFBQcDDTAKBggqhkjO
PQQDAwNpADBmAjEAmCPZnnlUQOKlcOIIOgFrRCkOqO0ESs+dobYwAc2rFCBtQyP7
C3N00xkX8WZZpiAZAjEAi1Z5+nGbJg5eJTc8fwudutN/HNwJEIS6mHds9kfcy26x
DAlVlhox680Jxy5J8Pkx
"""

    def setUp(self):
        self.asn1Spec = rfc5280.Certificate()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.cert_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.cert_pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        extn_list = []
        certificateExtensionsMap = opentypemap.get('certificateExtensionsMap')

        for extn in asn1Object['tbsCertificate']['extensions']:
            extn_list.append(extn['extnID'])
            if extn['extnID'] in rfc5280.certificateExtensionsMap.keys():
                extnValue, rest = der_decoder(extn['extnValue'],
                    asn1Spec=certificateExtensionsMap[extn['extnID']])
                self.assertFalse(rest)
                self.assertTrue(extnValue.prettyPrint())
                self.assertEqual(extn['extnValue'], der_encoder(extnValue))

                if extn['extnID'] == rfc4334.id_pe_wlanSSID:
                    self.assertIn(b'Example', extnValue)
            
                if extn['extnID'] == rfc5280.id_ce_extKeyUsage:
                    self.assertIn(rfc4334.id_kp_eapOverLAN, extnValue)
                    self.assertIn(rfc4334.id_kp_eapOverPPP, extnValue)

        self.assertIn(rfc4334.id_pe_wlanSSID, extn_list)
        self.assertIn(rfc5280.id_ce_extKeyUsage, extn_list)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite)



# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with assistance from the asn1ate tool, with manual
#   changes to AES_CCM_ICVlen.subtypeSpec and added comments
# Modified by Russ Housley to include the opentypemap manager, and drop use
#   of the _OID routine.
#
# Copyright (c) 2018-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
#  AES-CCM and AES-GCM Algorithms fo use with the Authenticated-Enveloped-Data
#  protecting content type for the Cryptographic Message Syntax (CMS)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5084.txt

from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import univ

from pyasn1_alt_modules import opentypemap

algorithmIdentifierMap = opentypemap.get('algorithmIdentifierMap')


# AES-CCM and AES-GCM Algorithm Paramaters

class AES_CCM_ICVlen(univ.Integer):
    pass


class AES_GCM_ICVlen(univ.Integer):
    pass


AES_CCM_ICVlen.subtypeSpec = constraint.SingleValueConstraint(4, 6, 8, 10, 12, 14, 16)

AES_GCM_ICVlen.subtypeSpec = constraint.ValueRangeConstraint(12, 16)


class CCMParameters(univ.Sequence):
    pass


CCMParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType('aes-nonce', univ.OctetString().subtype(subtypeSpec=constraint.ValueSizeConstraint(7, 13))),
    # The aes-nonce parameter contains 15-L octets, where L is the size of the length field. L=8 is RECOMMENDED.
    # Within the scope of any content-authenticated-encryption key, the nonce value MUST be unique.
    namedtype.DefaultedNamedType('aes-ICVlen', AES_CCM_ICVlen().subtype(value=12))
)


class GCMParameters(univ.Sequence):
    pass


GCMParameters.componentType = namedtype.NamedTypes(
    namedtype.NamedType('aes-nonce', univ.OctetString()),
    # The aes-nonce may have any number of bits between 8 and 2^64, but it MUST be a multiple of 8 bits.
    # Within the scope of any content-authenticated-encryption key, the nonce value MUST be unique.
    # A nonce value of 12 octets can be processed more efficiently, so that length is RECOMMENDED.
    namedtype.DefaultedNamedType('aes-ICVlen', AES_GCM_ICVlen().subtype(value=12))
)


# Object Identifiers

aes = univ.ObjectIdentifier((2, 16, 840, 1, 101, 3, 4, 1))

id_aes128_CCM = aes + (7,)

id_aes128_GCM = aes + (6,)

id_aes192_CCM = aes + (27,)

id_aes192_GCM = aes + (26,)

id_aes256_CCM = aes + (47,)

id_aes256_GCM = aes + (46,)


# Update the Algorithm Identifier Map and the S/MIME Capability Map

_algorithmIdentifierMapUpdate = {
    id_aes128_CCM: CCMParameters(),
    id_aes128_GCM: GCMParameters(),
    id_aes192_CCM: CCMParameters(),
    id_aes192_GCM: GCMParameters(),
    id_aes256_CCM: CCMParameters(),
    id_aes256_GCM: GCMParameters(),
}

algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2024-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Generating Transport Key Containers Using the GOST Algorithms
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9548.txt
#

from pyasn1_alt_modules import rfc9215


# Import from RFC 9215

GostR3410_2012_PublicKey = rfc9215.GostR3410_2012_PublicKey


# The Algorithm Identifier Map update was done by importing rfc9215.



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Clarification of RFC7030 CSR Attributes definition
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9908.txt
#

from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import namedval
from pyasn1.type import opentype
from pyasn1.type import tag
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import opentypemap

certificateAttributesMap = opentypemap.get('certificateAttributesMap')

MAX = float('inf')


# Imports from RFC 5280

Attribute = rfc5280.Attribute

AttributeType = rfc5280.AttributeType

AttributeValue = rfc5280.AttributeValue

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier


# NameTemplate
# like Name, but with OPTIONAL RDN values

class SingleAttributeTemplate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('type', AttributeType()),
        namedtype.OptionalNamedType('value', AttributeValue(),
            openType=opentype.OpenType('type', certificateAttributesMap)
        )
    )


class RelativeDistinguishedNameTemplate(univ.SetOf):
    componentType = SingleAttributeTemplate()
    sizeSpec = constraint.ValueSizeConstraint(1, MAX)


class RDNSequenceTemplate(univ.SequenceOf):
    componentType = RelativeDistinguishedNameTemplate()


class NameTemplate(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('rdnSequence', RDNSequenceTemplate())
    )


# SubjectPublicKeyInfoTemplate
# like SubjectPublicKeyInfo, but with OPTIONAL subjectPublicKey

class SubjectPublicKeyInfoTemplate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('algorithm', AlgorithmIdentifier()),
        namedtype.OptionalNamedType('subjectPublicKey', univ.BitString())
    )


# ExtensionTemplate
# like Extension, but with OPTIONAL extnValue

class ExtensionTemplate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('extnID', univ.ObjectIdentifier()),
        namedtype.DefaultedNamedType('critical', univ.Boolean().subtype(value=0)),
        namedtype.OptionalNamedType('extnValue', univ.OctetString())
    )


class ExtensionTemplates(univ.SequenceOf):
    componentType = ExtensionTemplate()
    sizeSpec = constraint.ValueSizeConstraint(1, MAX)


# Certification Request Info Template Attribute
# like CertificationRequestInfo, with OPTIONAL subject and subjectPKInfo 

id_aa = univ.ObjectIdentifier('1.2.840.113549.1.9.16.2')

id_aa_certificationRequestInfoTemplate = id_aa + (61, )


class Attributes(univ.SetOf):
    componentType = Attribute()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class CertificationRequestInfoTemplate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('version',
            univ.Integer(namedValues=namedval.NamedValues(('v1', 0)))),
        namedtype.OptionalNamedType('subject', NameTemplate()),
        namedtype.OptionalNamedType('subjectPKInfo',
            SubjectPublicKeyInfoTemplate().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.NamedType('attributes',
            Attributes().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 1)))
    )


aa_certificationRequestInfoTemplate = Attribute()
aa_certificationRequestInfoTemplate['type'] = id_aa_certificationRequestInfoTemplate
aa_certificationRequestInfoTemplate['values'][0] = CertificationRequestInfoTemplate()


# Extension Request Template Attribute
# like extensionRequest, but with OPTIONAL extnValues in each Extension

id_aa_extensionReqTemplate = id_aa + (62, )


class ExtensionReqTemplate(ExtensionTemplates):
    pass


aa_extensionReqTemplate = Attribute()
aa_extensionReqTemplate['type'] = id_aa_extensionReqTemplate
aa_extensionReqTemplate['values'][0] = ExtensionReqTemplate()


# Update the Certificate Attributes Map

_certificateMapUpdate = {
    id_aa_certificationRequestInfoTemplate: CertificationRequestInfoTemplate(),
    id_aa_extensionReqTemplate: ExtensionReqTemplate(),
}

certificateAttributesMap.update(_certificateMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Digital Signatures on geofeed data
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9092.txt
#

from pyasn1.type import univ

from pyasn1_alt_modules import opentypemap

cmsContentTypesMap = opentypemap.get('cmsContentTypesMap')


# CMS Content Type for Geofeed CSV with CRLF

id_ct = univ.ObjectIdentifier('1.2.840.113549.1.9.16.1')

id_ct_geofeedCSVwithCRLF = id_ct + (47, )


# Update the CMS Content Type Map

_cmsContentTypesMapUpdate = {
    id_ct_geofeedCSVwithCRLF: univ.OctetString(),
}

cmsContentTypesMap.update(_cmsContentTypesMapUpdate)



# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Edwards-Curve Digital Signature Algorithm (EdDSA) Signatures in the CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8419.txt
# https://www.rfc-editor.org/errata/eid5869

from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import opentypemap

algorithmIdentifierMap = opentypemap.get('algorithmIdentifierMap')


class ShakeOutputLen(univ.Integer):
    pass


id_Ed25519 = univ.ObjectIdentifier('1.3.101.112')

sigAlg_Ed25519 = rfc5280.AlgorithmIdentifier()
sigAlg_Ed25519['algorithm'] = id_Ed25519
# sigAlg_Ed25519['parameters'] is absent


id_Ed448 = univ.ObjectIdentifier('1.3.101.113')

sigAlg_Ed448 = rfc5280.AlgorithmIdentifier()
sigAlg_Ed448['algorithm'] = id_Ed448
# sigAlg_Ed448['parameters'] is absent


hashAlgs = univ.ObjectIdentifier('2.16.840.1.101.3.4.2')

id_sha512 = hashAlgs + (3, )

hashAlg_SHA_512 = rfc5280.AlgorithmIdentifier()
hashAlg_SHA_512['algorithm'] = id_sha512
# hashAlg_SHA_512['parameters'] is absent


id_shake256 = hashAlgs + (12, )

hashAlg_SHAKE256 = rfc5280.AlgorithmIdentifier()
hashAlg_SHAKE256['algorithm'] = id_shake256
# hashAlg_SHAKE256['parameters']is absent


id_shake256_len = hashAlgs + (18, )

hashAlg_SHAKE256_LEN  = rfc5280.AlgorithmIdentifier()
hashAlg_SHAKE256_LEN['algorithm'] = id_shake256_len
hashAlg_SHAKE256_LEN['parameters'] = ShakeOutputLen()


# Update the Algorithm Identifiers Map

_algorithmIdentifierMapUpdate = {
    id_shake256_len: ShakeOutputLen(),
}

algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with some assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Using Pre-Shared Key (PSK) in the Cryptographic Message Syntax (CMS)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8696.txt
#

from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import namedval
from pyasn1.type import tag
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import opentypemap

otherRecipientInfoMap = opentypemap.get('otherRecipientInfoMap')

MAX = float('inf')


id_ori = univ.ObjectIdentifier('1.2.840.113549.1.9.16.13')

id_ori_keyTransPSK = univ.ObjectIdentifier('1.2.840.113549.1.9.16.13.1')

id_ori_keyAgreePSK = univ.ObjectIdentifier('1.2.840.113549.1.9.16.13.2')


class PreSharedKeyIdentifier(univ.OctetString):
    pass


class KeyTransRecipientInfos(univ.SequenceOf):
    componentType = rfc5652.KeyTransRecipientInfo()


class KeyTransPSKRecipientInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('version',
            rfc5652.CMSVersion()),
        namedtype.NamedType('pskid',
            PreSharedKeyIdentifier()),
        namedtype.NamedType('kdfAlgorithm',
            rfc5652.KeyDerivationAlgorithmIdentifier()),
        namedtype.NamedType('keyEncryptionAlgorithm',
            rfc5652.KeyEncryptionAlgorithmIdentifier()),
        namedtype.NamedType('ktris',
            KeyTransRecipientInfos()),
        namedtype.NamedType('encryptedKey',
            rfc5652.EncryptedKey())
    )


class KeyAgreePSKRecipientInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('version',
            rfc5652.CMSVersion()),
        namedtype.NamedType('pskid',
            PreSharedKeyIdentifier()),
        namedtype.NamedType('originator',
            rfc5652.OriginatorIdentifierOrKey().subtype(explicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatConstructed, 0))),
        namedtype.OptionalNamedType('ukm',
            rfc5652.UserKeyingMaterial().subtype(explicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.NamedType('kdfAlgorithm',
            rfc5652.KeyDerivationAlgorithmIdentifier()),
        namedtype.NamedType('keyEncryptionAlgorithm',
            rfc5652.KeyEncryptionAlgorithmIdentifier()),
        namedtype.NamedType('recipientEncryptedKeys',
            rfc5652.RecipientEncryptedKeys())
    )


class CMSORIforPSKOtherInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('psk',
            univ.OctetString()),
        namedtype.NamedType('keyMgmtAlgType',
            univ.Enumerated(namedValues=namedval.NamedValues(
                ('keyTrans', 5), ('keyAgree', 10)))),
        namedtype.NamedType('keyEncryptionAlgorithm',
            rfc5652.KeyEncryptionAlgorithmIdentifier()),
        namedtype.NamedType('pskLength',
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(1, MAX))),
        namedtype.NamedType('kdkLength',
            univ.Integer().subtype(
                subtypeSpec=constraint.ValueRangeConstraint(1, MAX)))
    )


# Update the CMS Other Recipient Info Map

_otherRecipientInfoMapUpdate = {
    id_ori_keyTransPSK: KeyTransPSKRecipientInfo(),
    id_ori_keyAgreePSK: KeyAgreePSKRecipientInfo(),
}

otherRecipientInfoMap.update(_otherRecipientInfoMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder
from pyasn1.type import char
from pyasn1.type import univ

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc2986
from pyasn1_alt_modules import rfc5280


class CertificationRequestTestCase(unittest.TestCase):
    pem_text = """\
MIICxjCCAa4CAQAwgYAxCzAJBgNVBAYTAlVTMR0wGwYDVQQDDBRmY3UuZmFrZS5h
ZGRyZXNzLm9yZzEXMBUGA1UEBwwOUGxlYXNhbnQgR3JvdmUxHDAaBgNVBAoME0Zh
a2UgQ29tcGFueSBVbml0ZWQxDTALBgNVBAgMBFV0YWgxDDAKBgNVBAsMA0VuZzCC
ASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBALvnYesymhLKSm9Llix53BUA
h99xMDBUYk0OB1VIdNQyjmFabHinM+lYUzVzrfcm1xtYB5QYKbsYuwZ4r5WI7qho
CRJy6JwXqKpOe72ScCogxlGDr2QtKjtvyWrRwXBHX1/OqVSZ3hdz3njhKpmq6HgK
87vH26RCSmK8FqCgn+qePfpspA7GzBvYwXhXluQtG7r4yBMKNRTQlPst8Vcy+iK+
pI8hmQVrzGi8Hgbpr2L9EjPUOlAQEb8hxeKc7s5VhjN/RHMLVMX8YczZYt7mcDKr
3PMwOVmXL1DMCtnS50MA2AxcPWcbQBeGyMroP+DLhAt6y1/IT0H5sQruNQw4euMC
AwEAAaAAMA0GCSqGSIb3DQEBCwUAA4IBAQBQXYQPfH5Wy4o0ZFbKQOO1e3dHV8rl
e8m9Z6qLgJO8rtW+OI+4FavJ6zjUvNVzd9JJxgwQ/1xprwrXh36nPcSyNLpGs7JT
6u7TGQ38QQAOmziLXzauMWGBeLuzWGmOKA1cs5HFGLSmbxF3+0IWpz4GlD86pU1+
WYyWgWHHAMA+kFYwBUR6CvPkmhshnZ8vrQavoOlcidCJ8o6IGA7N/Z0/NrgIDcoz
YaruhoMrmRKHKNpfamhT0gvqEPBec+UB3uLElESIqaeqYc6eMtUQP3lqyghF6I0M
fi6h7i9VVAZpslaKFfkNg12gLbbsCB1q36l5VXjHY/qe0FIUa9ogRrOi
"""

    def setUp(self):
        self.asn1Spec = rfc2986.CertificationRequest()

    def testDerCodec(self):

        substrate = pem.readBase64fromText(self.pem_text)

        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

    def testOpenTypes(self):
        openTypesMap = {
            univ.ObjectIdentifier('1.2.840.113549.1.1.1'): univ.Null(""),
            univ.ObjectIdentifier('1.2.840.113549.1.1.5'): univ.Null(""),
            univ.ObjectIdentifier('1.2.840.113549.1.1.11'): univ.Null(""),
        }

        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate,
                                       asn1Spec=rfc2986.CertificationRequest(),
                                       openTypes=openTypesMap,
                                       decodeOpenTypes=True)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        for rdn in asn1Object['certificationRequestInfo']['subject']['rdnSequence']:
            for atv in rdn:
                if atv['type'] == rfc5280.id_at_countryName:
                    self.assertEqual(char.PrintableString('US'), atv['value'])

                else:
                    self.assertGreater(len(atv['value']['utf8String']), 2)

        spki_alg = asn1Object['certificationRequestInfo']['subjectPKInfo']['algorithm']

        self.assertEqual(univ.Null(""), spki_alg['parameters'])

        sig_alg = asn1Object['signatureAlgorithm']

        self.assertEqual(univ.Null(""), sig_alg['parameters'])


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Enhanced JWT Claim Constraints certificate extensions
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9118.txt

from pyasn1.type import char
from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import tag
from pyasn1.type import univ

from pyasn1_alt_modules import opentypemap

certificateExtensionsMap = opentypemap.get('certificateExtensionsMap')

MAX = float('inf')


# EnhancedJWTClaimConstraints Certificate Extension

id_pe_eJWTClaimConstraints = univ.ObjectIdentifier('1.3.6.1.5.5.7.1.33')


class JWTClaimName(char.IA5String):
    pass


class JWTClaimNames(univ.SequenceOf):
    componentType = JWTClaimName()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class JWTClaimValues(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('claim', JWTClaimName()),
        namedtype.NamedType('values', univ.SequenceOf(
            componentType=char.UTF8String()).subtype(
                sizeSpec=constraint.ValueSizeConstraint(1, MAX)))
    )


class JWTClaimValuesList(univ.SequenceOf):
    componentType = JWTClaimValues()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class EnhancedJWTClaimConstraints(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('mustInclude',
            JWTClaimNames().subtype(explicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.OptionalNamedType('permittedValues',
            JWTClaimValuesList().subtype(explicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.OptionalNamedType('mustExclude',
            JWTClaimNames().subtype(explicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 2)))
    )
    subtypeSpec = constraint.ConstraintsUnion(
        constraint.WithComponentsConstraint(
            ('mustInclude', constraint.ComponentPresentConstraint())),
        constraint.WithComponentsConstraint(
            ('permittedValues', constraint.ComponentPresentConstraint())),
        constraint.WithComponentsConstraint(
            ('mustExclude', constraint.ComponentPresentConstraint()))
    )


# Update the Certificate Extension Map

_certificateExtensionsMapUpdate = {
    id_pe_eJWTClaimConstraints: EnhancedJWTClaimConstraints(),
}

certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5275
from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import rfc6402


class GLUseKEKTestCase(unittest.TestCase):
    pem_text = """\

KS1zJZvX/ury+ySGvKDLkfnqwZARR9W7TkTdx0L9W9oVjyEgOeGkvA==
"""

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)

        layers = { }
        layers.update(rfc5652.cmsContentTypesMap)

        getNextLayer = {
            rfc5652.id_ct_contentInfo: lambda x: x['contentType'],
            rfc5652.id_signedData: lambda x: x['encapContentInfo']['eContentType'],
            rfc6402.id_cct_PKIData: lambda x: None
        }

        getNextSubstrate = {
            rfc5652.id_ct_contentInfo: lambda x: x['content'],
            rfc5652.id_signedData: lambda x: x['encapContentInfo']['eContent'],
            rfc6402.id_cct_PKIData: lambda x: None
        }

        next_layer = rfc5652.id_ct_contentInfo
        while next_layer:
            asn1Object, rest = der_decoder(
                substrate, asn1Spec=layers[next_layer])

            self.assertFalse(rest)
            self.assertTrue(asn1Object.prettyPrint())
            self.assertEqual(substrate, der_encoder(asn1Object))

            substrate = getNextSubstrate[next_layer](asn1Object)
            next_layer = getNextLayer[next_layer](asn1Object)

        found_gl_use_kek = False
        for ctrl in asn1Object['controlSequence']:
            if ctrl['attrType'] == rfc5275.id_skd_glUseKEK:
                cv, rest = der_decoder(
                    ctrl['attrValues'][0],
                    asn1Spec=rfc5652.cmsAttributesMap[ctrl['attrType']])

                self.assertFalse(rest)
                self.assertTrue(cv.prettyPrint())
                self.assertEqual(ctrl['attrValues'][0], der_encoder(cv))

                self.assertIn(
                    'example.com',
                    cv['glInfo']['glAddress']['rfc822Name'])

                self.assertIn(
                    'example.com',
                    cv['glOwnerInfo'][0]['glOwnerAddress']['rfc822Name'])

                self.assertEqual(31, cv['glKeyAttributes']['duration'])
                found_gl_use_kek = True

        self.assertTrue(found_gl_use_kek)

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=rfc5652.ContentInfo(), decodeOpenTypes=True)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        sd = asn1Object['content']
        self.assertEqual(
            rfc6402.id_cct_PKIData, sd['encapContentInfo']['eContentType'])

        pkid, rest = der_decoder(
            sd['encapContentInfo']['eContent'],
            asn1Spec=rfc6402.PKIData(),
            decodeOpenTypes=True)

        self.assertFalse(rest)
        self.assertTrue(pkid.prettyPrint())
        self.assertEqual(sd['encapContentInfo']['eContent'], der_encoder(pkid))

        found_gl_use_kek = False
        for ctrl in pkid['controlSequence']:
            if ctrl['attrType'] == rfc5275.id_skd_glUseKEK:
                cv = ctrl['attrValues'][0]

                self.assertIn(
                    'example.com',
                    cv['glInfo']['glAddress']['rfc822Name'])

                self.assertIn(
                    'example.com',
                    cv['glOwnerInfo'][0]['glOwnerAddress']['rfc822Name'])

                self.assertEqual(31, cv['glKeyAttributes']['duration'])
                found_gl_use_kek = True

        self.assertTrue(found_gl_use_kek)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    import sys

    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der import decoder as der_decoder
from pyasn1.codec.der import encoder as der_encoder
from pyasn1.type import univ

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc4055


class PSSDefautTestCase(unittest.TestCase):
    pss_default_pem_text = "MAsGCSqGSIb3DQEBCg=="

    def setUp(self):
        self.asn1Spec = rfc5280.AlgorithmIdentifier()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pss_default_pem_text)
        asn1Object, rest = der_decoder.decode(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(rfc4055.id_RSASSA_PSS, asn1Object[0])
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pss_default_pem_text)
        asn1Object, rest = der_decoder.decode(substrate,
                                              asn1Spec=self.asn1Spec,
                                              decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertFalse(asn1Object['parameters'].hasValue())


class PSSSHA512TestCase(unittest.TestCase):
    pss_sha512_pem_text = "MDwGCSqGSIb3DQEBCjAvoA8wDQYJYIZIAWUDBAIDBQChHDAaBg" \
                          "kqhkiG9w0BAQgwDQYJYIZIAWUDBAIDBQA="

    def setUp(self):
        self.asn1Spec = rfc5280.AlgorithmIdentifier()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pss_sha512_pem_text)
        asn1Object, rest = der_decoder.decode(
            substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(rfc4055.id_RSASSA_PSS, asn1Object[0])
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.pss_sha512_pem_text)
        asn1Object, rest = der_decoder.decode(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertTrue(asn1Object['parameters'].hasValue())
        self.assertTrue(20, asn1Object['parameters']['saltLength'])


class OAEPDefautTestCase(unittest.TestCase):
    oaep_default_pem_text = "MAsGCSqGSIb3DQEBBw=="

    def setUp(self):
        self.asn1Spec = rfc5280.AlgorithmIdentifier()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.oaep_default_pem_text)
        asn1Object, rest = der_decoder.decode(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(rfc4055.id_RSAES_OAEP, asn1Object[0])
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.oaep_default_pem_text)
        asn1Object, rest = der_decoder.decode(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertFalse(asn1Object['parameters'].hasValue())


class OAEPSHA256TestCase(unittest.TestCase):
    oaep_sha256_pem_text = "MDwGCSqGSIb3DQEBBzAvoA8wDQYJYIZIAWUDBAIBBQChHDAaB" \
                           "gkqhkiG9w0BAQgwDQYJYIZIAWUDBAIBBQA="

    def setUp(self):
        self.asn1Spec = rfc5280.AlgorithmIdentifier()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.oaep_sha256_pem_text)
        asn1Object, rest = der_decoder.decode(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(rfc4055.id_RSAES_OAEP, asn1Object[0])
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.oaep_sha256_pem_text)
        asn1Object, rest = der_decoder.decode(
            substrate, asn1Spec=self.asn1Spec, decodeOpenTypes=True)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder.encode(asn1Object))
        self.assertTrue(asn1Object['parameters'].hasValue())

        oaep_p = asn1Object['parameters']

        self.assertEqual(univ.Null(""), oaep_p['hashFunc']['parameters'])
        self.assertEqual(univ.Null(""), oaep_p['maskGenFunc']['parameters']['parameters'])


class OAEPFullTestCase(unittest.TestCase):
    oaep_full_pem_text = "MFMGCSqGSIb3DQEBBzBGoA8wDQYJYIZIAWUDBAICBQChHDAaBgk" \
                         "qhkiG9w0BAQgwDQYJYIZIAWUDBAICBQCiFTATBgkqhkiG9w0BAQ" \
                         "kEBmZvb2Jhcg=="

    def setUp(self):
        self.asn1Spec = rfc5280.AlgorithmIdentifier()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.oaep_full_pem_text)
        asn1Object, rest = der_decoder.decode(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(rfc4055.id_RSAES_OAEP, asn1Object[0])
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

    def testOpenTypes(self):
        substrate = pem.readBase64fromText(self.oaep_full_pem_text)
        asn1Object, rest = der_decoder.decode(substrate,
                                              asn1Spec=self.asn1Spec,
                                              decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder.encode(asn1Object))

        self.assertTrue(asn1Object['parameters'].hasValue())

        oaep_p = asn1Object['parameters']

        self.assertEqual(univ.Null(""), oaep_p['hashFunc']['parameters'])
        self.assertEqual(
            univ.Null(""), oaep_p['maskGenFunc']['parameters']['parameters'])
        self.assertEqual(
            univ.OctetString(value='foobar'),
            oaep_p['pSourceFunc']['parameters'])


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Certificate Store Access via HTTP
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc4387.txt
#


from pyasn1.type import univ


id_ad = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 48, ))

id_ad_http_certs = id_ad + (6, )

id_ad_http_crls = id_ad  + (7,)



# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with assistance from the asn1ate tool.
# Auto-generated by asn1ate v.0.6.0 from rfc8103.asn.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# ChaCha20Poly1305 algorithm fo use with the Authenticated-Enveloped-Data
# protecting content type for the Cryptographic Message Syntax (CMS)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8103.txt

from pyasn1.type import constraint
from pyasn1.type import univ


def _OID(*components):
    output = []
    for x in tuple(components):
        if isinstance(x, univ.ObjectIdentifier):
            output.extend(list(x))
        else:
            output.append(int(x))

    return univ.ObjectIdentifier(output)


class AEADChaCha20Poly1305Nonce(univ.OctetString):
    pass


AEADChaCha20Poly1305Nonce.subtypeSpec = constraint.ValueSizeConstraint(12, 12)

id_alg_AEADChaCha20Poly1305 = _OID(1, 2, 840, 113549, 1, 9, 16, 3, 18)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Certificate Transparency
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc6962.txt
#

from pyasn1.type import univ

from pyasn1_alt_modules import opentypemap

certificateExtensionsMap = opentypemap.get('certificateExtensionsMap')


class SignedCertificateTimestampList(univ.OctetString):
    pass


id_ce_embeddedSCT = univ.ObjectIdentifier('1.3.6.1.4.1.11129.2.4.2')


id_ce_criticalPoison = univ.ObjectIdentifier('1.3.6.1.4.1.11129.2.4.3')


id_kp_precertificateSigning = univ.ObjectIdentifier('1.3.6.1.4.1.11129.2.4.4')


id_pkix_ocsp_SCT = univ.ObjectIdentifier('1.3.6.1.4.1.11129.2.4.5')


# Update the Certificate Extension Map
#
# Note that rfc6960.py also uses this same map for OCSP extensions.
# The id_ce_criticalPoison OID is not automatically added to the map
# because normal relying parties are supposed to reject certificates
# that contain it.

_certificateExtensionsMapUpdate = {
    id_ce_embeddedSCT: SignedCertificateTimestampList(),
    # id_ce_criticalPoison: univ.Null(""),
    id_pkix_ocsp_SCT: SignedCertificateTimestampList(),
}

certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
#
# Copyright (c) 2024-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# HSS/LMS Hash-based Signature Algorithm for CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfcXXXX.txt

from pyasn1_alt_modules import rfc8708


# Object Identifiers

id_alg_hss_lms_hashsig = rfc8708.id_alg_hss_lms_hashsig

id_alg_mts_hashsig = id_alg_hss_lms_hashsig


# Signature Algorithm Identifier

sa_HSS_LMS_HashSig = rfc8708.sa_HSS_LMS_HashSig
# sa_HSS_LMS_HashSig['parameters'] is alway absent


# Public Key

HSS_LMS_HashSig_PublicKey = rfc8708.HSS_LMS_HashSig_PublicKey

pk_HSS_LMS_HashSig = rfc8708.pk_HSS_LMS_HashSig
# pk_HSS_LMS_HashSig['subjectPublicKey'] CONTAINS the
#     HSS/LMS public key without any ASN.1 encoding



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# RSASSA-PSS Signature Algorithm in CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc4056.txt
#

from pyasn1_alt_modules import rfc4055


# Imports from RFC 4055

rsaEncryption = rfc4055.rsaEncryption

id_RSASSA_PSS = rfc4055.id_RSASSA_PSS

RSAPublicKey = rfc4055.RSAPublicKey

RSASSA_PSS_params = rfc4055.RSASSA_PSS_params

rSASSA_PSS_Default_Params = rfc4055.rSASSA_PSS_Default_Params

rSASSA_PSS_Default_Identifier = rfc4055.rSASSA_PSS_Default_Identifier

rSASSA_PSS_SHA224_Params = rfc4055.rSASSA_PSS_SHA224_Params

rSASSA_PSS_SHA224_Identifier = rfc4055.rSASSA_PSS_SHA224_Identifier

rSASSA_PSS_SHA256_Params = rfc4055.rSASSA_PSS_SHA256_Params

rSASSA_PSS_SHA256_Identifier = rfc4055.rSASSA_PSS_SHA256_Identifier

rSASSA_PSS_SHA384_Params = rfc4055.rSASSA_PSS_SHA384_Params

rSASSA_PSS_SHA384_Identifier = rfc4055.rSASSA_PSS_SHA384_Identifier

rSASSA_PSS_SHA512_Params = rfc4055.rSASSA_PSS_SHA512_Params

rSASSA_PSS_SHA512_Identifier = rfc4055.rSASSA_PSS_SHA512_Identifier


# The Algorithm Identifiers Map is updated by importing rfc4055.
#  As a reminder it includes:
#
# _algorithmIdentifierMapUpdate = {
#     id_RSASSA_PSS: RSASSA_PSS_params(),
# }
# 
# algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Attribute Certificate Policies Extension
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc4476.txt
#

from pyasn1.type import char
from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import opentypemap

certificateExtensionsMap = opentypemap.get('certificateExtensionsMap')

policyQualifierInfosMap = opentypemap.get('policyQualifierInfosMap')

MAX = float('inf')


# Imports from RFC 5280

PolicyQualifierId = rfc5280.PolicyQualifierId

PolicyQualifierInfo = rfc5280.PolicyQualifierInfo

UserNotice = rfc5280.UserNotice

id_pkix = rfc5280.id_pkix


# Object Identifiers

id_pe = id_pkix + (1,)

id_pe_acPolicies = id_pe + (15,)

id_qt = id_pkix + (2,)

id_qt_acps = id_qt + (4,)

id_qt_acunotice = id_qt + (5,)


# Attribute Certificate Policies Extension

class ACUserNotice(UserNotice):
    pass


class ACPSuri(char.IA5String):
    pass


class AcPolicyId(univ.ObjectIdentifier):
    pass


class PolicyInformation(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('policyIdentifier', AcPolicyId()),
        namedtype.OptionalNamedType('policyQualifiers',
            univ.SequenceOf(componentType=PolicyQualifierInfo()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)))
    )


class AcPoliciesSyntax(univ.SequenceOf):
    componentType = PolicyInformation()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


# Update the policy qualifier map

_policyQualifierInfosMapUpdate = {
    id_qt_acps: ACPSuri(),
    id_qt_acunotice: UserNotice(),
}

policyQualifierInfosMap.update(_policyQualifierInfosMapUpdate)


# Update the certificate extension map

_certificateExtensionsMapUpdate = {
    id_pe_acPolicies: AcPoliciesSyntax(),
}

certificateExtensionsMap.update(_certificateExtensionsMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Enrollment over Secure Transport (EST)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc7030.txt
#

from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import opentypemap

cmsAttributesMap = opentypemap.get('cmsAttributesMap')

MAX = float('inf')


# Imports from RFC 5652

Attribute = rfc5652.Attribute


# Asymmetric Decrypt Key Identifier Attribute

id_aa_asymmDecryptKeyID = univ.ObjectIdentifier('1.2.840.113549.1.9.16.2.54')

class AsymmetricDecryptKeyIdentifier(univ.OctetString):
    pass


aa_asymmDecryptKeyID = Attribute()
aa_asymmDecryptKeyID['attrType'] = id_aa_asymmDecryptKeyID
aa_asymmDecryptKeyID['attrValues'][0] = AsymmetricDecryptKeyIdentifier()


# CSR Attributes

class AttrOrOID(univ.Choice):
    pass

AttrOrOID.componentType = namedtype.NamedTypes(
    namedtype.NamedType('oid', univ.ObjectIdentifier()),
    namedtype.NamedType('attribute', Attribute())
)


class CsrAttrs(univ.SequenceOf):
    pass

CsrAttrs.componentType = AttrOrOID()
CsrAttrs.subtypeSpec=constraint.ValueSizeConstraint(0, MAX)

   
# Update CMS Attribute Map

_cmsAttributesMapUpdate = {
    id_aa_asymmDecryptKeyID: AsymmetricDecryptKeyIdentifier(),
}

cmsAttributesMap.update(_cmsAttributesMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
# Modified by Russ Housley to add synonyms with two digit years for
#   some of the OIDs to align with the ASN.1 module in RFC 9215.
#
# Copyright (c) 2022-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# GOST R 34.10-2012 and GOST R 34.11-2012 Algorithms
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9215.txt
#

from pyasn1.type import char
from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import namedval
from pyasn1.type import univ

from pyasn1_alt_modules import opentypemap

algorithmIdentifierMap = opentypemap.get('algorithmIdentifierMap')

certificateAttributesMap = opentypemap.get('certificateAttributesMap')

certificateExtensionsMap = opentypemap.get('certificateExtensionsMap')


# MODULE: GostR3410-2012-PKISyntax { 1 2 643 7 1 0 2 }

id_tc26 = univ.ObjectIdentifier((1, 2, 643, 7, 1))

id_tc26_sign = id_tc26 + (1, 1)

id_tc26_digest = id_tc26 + (1, 2)

id_tc26_sign_constants = id_tc26 + (2, 1)

id_tc26_gost_3410_2012_256_constants = id_tc26_sign_constants + (1,)

id_tc26_gost_3410_2012_512_constants = id_tc26_sign_constants + (2,)

id_tc26_gost3410_2012_256 = id_tc26_sign + (1,)

id_tc26_gost3410_12_256 = id_tc26_gost3410_2012_256

id_tc26_gost3410_2012_512 = id_tc26_sign + (2,)

id_tc26_gost3410_12_512 = id_tc26_gost3410_2012_512

id_tc26_gost3411_12_256 = id_tc26_digest + (2,)

id_tc26_gost3411_12_512 = id_tc26_digest + (3,)

id_tc26_signwithdigest = id_tc26 + (1, 3)

id_tc26_signwithdigest_gost3410_2012_256 = id_tc26_signwithdigest + (2,)

id_tc26_signwithdigest_gost3410_12_256 = id_tc26_signwithdigest_gost3410_2012_256

id_tc26_signwithdigest_gost3410_2012_512 = id_tc26_signwithdigest + (3,)

id_tc26_signwithdigest_gost3410_12_512 = id_tc26_signwithdigest_gost3410_2012_512

id_tc26_gost_3410_2012_256_paramSetA = id_tc26_gost_3410_2012_256_constants + (1,)

id_tc26_gost_3410_2012_256_paramSetB = id_tc26_gost_3410_2012_256_constants + (2,)

id_tc26_gost_3410_2012_256_paramSetC = id_tc26_gost_3410_2012_256_constants + (3,)

id_tc26_gost_3410_2012_256_paramSetD = id_tc26_gost_3410_2012_256_constants + (4,)

id_tc26_gost_3410_2012_512_paramSetTest = id_tc26_gost_3410_2012_512_constants + (0,)

id_tc26_gost_3410_2012_512_paramSetA = id_tc26_gost_3410_2012_512_constants + (1,)

id_tc26_gost_3410_2012_512_paramSetB = id_tc26_gost_3410_2012_512_constants + (2,)

id_tc26_gost_3410_2012_512_paramSetC = id_tc26_gost_3410_2012_512_constants + (3,)


class GostR3410_2012_256_PublicKey(univ.OctetString):
    subtypeSpec = constraint.ValueSizeConstraint(64, 64)


class GostR3410_2012_512_PublicKey(univ.OctetString):
    subtypeSpec = constraint.ValueSizeConstraint(128, 128)


class GostR3410_2012_PublicKey(univ.OctetString):
    subtypeSpec = constraint.ConstraintsUnion(
        constraint.ValueSizeConstraint(64, 64),
        constraint.ValueSizeConstraint(128, 128)
    )


class GostR3410_2012_PublicKeyParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('publicKeyParamSet', univ.ObjectIdentifier()),
        namedtype.OptionalNamedType('digestParamSet', univ.ObjectIdentifier())
    )


# MODULE: RuStrongCertsSyntax { 1 2 643 7 1 0 6 }

id_ca = univ.ObjectIdentifier((1, 2, 643, 3))

id_fss = univ.ObjectIdentifier((1, 2, 643, 100))

id_fns = id_ca + (131,)


class OGRN(char.NumericString):
    subtypeSpec = constraint.ValueSizeConstraint(13, 13)

id_OGRN = id_fss + (1,)


class SNILS(char.NumericString):
    subtypeSpec = constraint.ValueSizeConstraint(11, 11)

id_SNILS = id_fss + (3,)


class OGRNIP(char.NumericString):
    subtypeSpec = constraint.ValueSizeConstraint(15, 15)

id_OGRNIP = id_fss + (5,)


id_class = id_fss + (113,)

id_class_kc1 = id_class + (1,)

id_class_kc2 = id_class + (2,)

id_class_kc3 = id_class + (3,)

id_class_kb1 = id_class + (4,)

id_class_kb2 = id_class + (5,)

id_class_ka = id_class + (6,)


class INN(char.NumericString):
    subtypeSpec = constraint.ValueSizeConstraint(12, 12)

id_INN = id_fns + (1, 1)


class INNLE(char.NumericString):
    subtypeSpec = constraint.ValueSizeConstraint(10, 10)

id_INNLE = id_fss + (4,)


class SubjectSignTool(char.UTF8String):
    subtypeSpec = constraint.ValueSizeConstraint(1, 200)

id_SubjectSignTool = id_fss + (111,)


class IssuerSignTool(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('signTool',
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 200))),
        namedtype.NamedType('cATool',
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 200))),
        namedtype.NamedType('signToolCert',
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 100))),
        namedtype.NamedType('cAToolCert',
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 100)))
    )

id_IssuerSignTool = id_fss + (112,)


class IdentificationKind(univ.Integer):
    namedValues = namedval.NamedValues(
        ('personal', 0),
        ('remote_cert', 1),
        ('remote_passport', 2),
        ('remote_system', 3)
    )

id_IdentificationKind = id_fss + (114,)


# Update the Algorithm Identifier Map

_algorithmIdentifierMapUpdate = {
    id_tc26_gost3410_2012_256: GostR3410_2012_PublicKeyParameters(),
    id_tc26_gost3410_2012_512: GostR3410_2012_PublicKeyParameters(),
}

algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)


# Update the Certificate Attribute Map

_certificateAttributesMapUpdate = {
    id_INN: INN(),
    id_INNLE: INNLE(),
    id_OGRN: OGRN(),
    id_OGRNIP: OGRNIP(),
    id_SNILS: SNILS(),
    id_IdentificationKind: IdentificationKind()
}

certificateAttributesMap.update(_certificateAttributesMapUpdate)


# Update the Certificate Extension Map

_certificateExtensionsMap = {
    id_SubjectSignTool: SubjectSignTool(),
    id_IssuerSignTool: IssuerSignTool(),
}

certificateExtensionsMap.update(_certificateExtensionsMap)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc3161


class TSPQueryTestCase(unittest.TestCase):
    tsp_query_pem_text = """\
MFYCAQEwUTANBglghkgBZQMEAgMFAARAGu1DauxDZZv8F7l4EKIbS00U40mUKfBW5C0giEz0
t1zOHCvK4A8i8zxwUXFHv4pAJZE+uFhZ+v53HTg9rLjO5Q==
"""

    def setUp(self):
        self.asn1Spec = rfc3161.TimeStampReq()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.tsp_query_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))


class TSPResponseTestCase(unittest.TestCase):
    tsp_response_pem_text = """\
MIIFMTADAgEAMIIFKAYJKoZIhvcNAQcCoIIFGTCCBRUCAQMxCzAJBgUrDgMCGgUAMIIBowYL
KoZIhvcNAQkQAQSgggGSBIIBjjCCAYoCAQEGBCoDBAEwUTANBglghkgBZQMEAgMFAARAGu1D
auxDZZv8F7l4EKIbS00U40mUKfBW5C0giEz0t1zOHCvK4A8i8zxwUXFHv4pAJZE+uFhZ+v53
HTg9rLjO5QIDDwJEGA8yMDE5MDUxMDE4MzQxOFoBAf+gggERpIIBDTCCAQkxETAPBgNVBAoT
CEZyZWUgVFNBMQwwCgYDVQQLEwNUU0ExdjB0BgNVBA0TbVRoaXMgY2VydGlmaWNhdGUgZGln
aXRhbGx5IHNpZ25zIGRvY3VtZW50cyBhbmQgdGltZSBzdGFtcCByZXF1ZXN0cyBtYWRlIHVz
aW5nIHRoZSBmcmVldHNhLm9yZyBvbmxpbmUgc2VydmljZXMxGDAWBgNVBAMTD3d3dy5mcmVl
dHNhLm9yZzEiMCAGCSqGSIb3DQEJARYTYnVzaWxlemFzQGdtYWlsLmNvbTESMBAGA1UEBxMJ
V3VlcnpidXJnMQswCQYDVQQGEwJERTEPMA0GA1UECBMGQmF5ZXJuMYIDWjCCA1YCAQEwgaMw
gZUxETAPBgNVBAoTCEZyZWUgVFNBMRAwDgYDVQQLEwdSb290IENBMRgwFgYDVQQDEw93d3cu
ZnJlZXRzYS5vcmcxIjAgBgkqhkiG9w0BCQEWE2J1c2lsZXphc0BnbWFpbC5jb20xEjAQBgNV
BAcTCVd1ZXJ6YnVyZzEPMA0GA1UECBMGQmF5ZXJuMQswCQYDVQQGEwJERQIJAMHphhYNqOmC
MAkGBSsOAwIaBQCggYwwGgYJKoZIhvcNAQkDMQ0GCyqGSIb3DQEJEAEEMBwGCSqGSIb3DQEJ
BTEPFw0xOTA1MTAxODM0MThaMCMGCSqGSIb3DQEJBDEWBBSuLICty7PQHx0Ynk0a3rGcCRrf
EjArBgsqhkiG9w0BCRACDDEcMBowGDAWBBSRbaPYYOzKguNLxZ0Xk+fpaIdfFDANBgkqhkiG
9w0BAQEFAASCAgBFDVbGQ3L5GcaUBMtBnMW7x3S57QowQhhrTewvncY+3Nc2i6tlM1UEdxIp
3m2iMqaH/N2xIm2sU/L/lIwaT1XIS4bJ2Nn8UPjZu/prJrVUFTMjJ5LWkG55x6c5A4pa2xxS
N/kOV2e+6RHYlGvcDOvu2fzuz08hE+NjaHIPg3idU1cBsl0gTWZCTrxdXTLuuvHahxUAdQKm
gTdGPjIiOR4GYpaVxEAgulaBQLZU5MhfBTASI1LkljhiFeDBQMhTUeZoA59/OxgnQR1Zpca4
ZuWuqnZImxziRQA1tX/6pjAo5eP1V+SLWYHeIO7ia/urGIK9AXd3jY3Ljq4h7R1E+RRKIseO
74mmtbJtCaiGL9H+6k164qC7U5fHBzKl3UboZtOUmNj10IJPUNyKQ5JPwCe6HEhbeXLRdh/8
bjdqy56hBHyG1NRBqiTXTvj9LOzsJGIF5GjwyCT0B2hpvzdTdzNtfQ27HUUYgnYg0fGEpNpi
vyaW5qCh9S704IKB0m/fXlqiIfNVdqDr/aAHNww8CouZP2oFO61WXCspbFNPLubeqxd5P4o4
dJzD4PKsurILdX7SL8pRI+O2UtJLwNB1t3LBLKfTZuOWoSBFvQwbqBsDEchrZIDZXSXMbXd6
uuvuO3ZsRWuej+gso+nWi3CRnRc9Wb0++cq4s8YSLaYSj2pHMA==
"""

    def setUp(self):
        self.asn1Spec = rfc3161.TimeStampResp()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.tsp_response_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import rfc5636
from pyasn1_alt_modules import opentypemap


class TraceableAnonymousCertificateTestCase(unittest.TestCase):
    pem_text = """\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"""

    def testDerCodec(self):

        substrate = pem.readBase64fromText(self.pem_text)

        layers = { }
        layers.update(opentypemap.get('cmsContentTypesMap'))

        getNextLayer = {
            rfc5652.id_ct_contentInfo: lambda x: x['contentType'],
            rfc5652.id_signedData: lambda x: x['encapContentInfo']['eContentType'],
            rfc5636.id_kisa_tac_token: lambda x: None
        }

        getNextSubstrate = {
            rfc5652.id_ct_contentInfo: lambda x: x['content'],
            rfc5652.id_signedData: lambda x: x['encapContentInfo']['eContent'],
            rfc5636.id_kisa_tac_token: lambda x: None
        }

        next_layer = rfc5652.id_ct_contentInfo
        while next_layer:
            asn1Object, rest = der_decoder(
                substrate, asn1Spec=layers[next_layer])

            self.assertFalse(rest)
            self.assertTrue(asn1Object.prettyPrint())
            self.assertEqual(substrate, der_encoder(asn1Object))

            substrate = getNextSubstrate[next_layer](asn1Object)
            next_layer = getNextLayer[next_layer](asn1Object)

        self.assertEqual('2019', asn1Object['timeout'][:4])
        self.assertEqual('5dcdf44e', asn1Object['userKey'].prettyPrint()[-8:])

    def testOpenTypes(self):
        asn1Spec=rfc5652.ContentInfo()
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(
            substrate, asn1Spec=asn1Spec, decodeOpenTypes=True)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        substrate = asn1Object['content']['encapContentInfo']['eContent']
        oid = asn1Object['content']['encapContentInfo']['eContentType']
        cmsContentTypesMap = opentypemap.get('cmsContentTypesMap')
        self.assertIn(oid, cmsContentTypesMap)

        tac_token, rest = der_decoder(
            substrate,
            asn1Spec=cmsContentTypesMap[oid],
            decodeOpenTypes=True)

        self.assertFalse(rest)
        self.assertTrue(tac_token.prettyPrint())
        self.assertEqual(substrate, der_encoder(tac_token))

        self.assertEqual('2019', tac_token['timeout'][:4])
        self.assertEqual('5dcdf44e', tac_token['userKey'].prettyPrint()[-8:])


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2022-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc9336


class DocSigningCertificateTestCase(unittest.TestCase):
    cert_pem_text = """\
MIIClDCCAhmgAwIBAgIUFXu5hcVxmT/6ufi0cmwGouTF7YEwCgYIKoZIzj0EAwMw
PzELMAkGA1UEBhMCVVMxCzAJBgNVBAgMAlZBMRAwDgYDVQQHDAdIZXJuZG9uMREw
DwYDVQQKDAhCb2d1cyBDQTAeFw0yMjA4MTEwMDAwMjRaFw0yMzA4MTEwMDAwMjRa
MEwxCzAJBgNVBAYTAlVTMQswCQYDVQQIEwJWQTEQMA4GA1UEBxMHSGVybmRvbjEQ
MA4GA1UEChMHRXhhbXBsZTEMMAoGA1UEAxMDQm9iMHYwEAYHKoZIzj0CAQYFK4EE
ACIDYgAEMaRiVS8WvN8Ycmpfq75jBbOMUukNfXAg6AL0JJBXtIFAuIJcZVlkLn/x
bywkcMLHK/O+w9RWUQa2Cjw+h8b/1Cl+gIpqLtE558bD5PfM2aYpJ/YE6yZ9nBfT
Qs7z1TH5o4HIMIHFMA4GA1UdDwEB/wQEAwIHgDBCBglghkgBhvhCAQ0ENRYzVGhp
cyBjZXJ0aWZpY2F0ZSBjYW5ub3QgYmUgdHJ1c3RlZCBmb3IgYW55IHB1cnBvc2Uu
MB0GA1UdDgQWBBTKa2Zy3iybV3+YjuLDKtNmjsIapTAfBgNVHSMEGDAWgBTyNds0
BNqlVfK9aQOZsGLs4hUIwTAaBgNVHREEEzARgQ9ib2JAZXhhbXBsZS5jb20wEwYD
VR0lBAwwCgYIKwYBBQUHAyQwCgYIKoZIzj0EAwMDaQAwZgIxAMEMwE4HoI1EbTKB
cKalSvfmChu8pixAOHHuCqMls/xupjIvH7tYphI/MaO9QxBKBgIxAPsr8XyO0VD7
mqtU35ik+eawW2Nf2x+gWPB8gS6jvQDBYV8N97ORHSH+cRuguuSBFg==
"""

    def setUp(self):
        self.asn1Spec = rfc5280.Certificate()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.cert_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        found = False
        for extn in asn1Object['tbsCertificate']['extensions']:
            if extn['extnID'] in rfc5280.certificateExtensionsMap:
                extnValue, rest = der_decoder(
                    extn['extnValue'],
                    asn1Spec=rfc5280.certificateExtensionsMap[extn['extnID']])
                self.assertFalse(rest)
                self.assertEqual(extn['extnValue'], der_encoder(extnValue))

                if extn['extnID'] == rfc5280.id_ce_extKeyUsage:
                    self.assertEqual(
                        rfc9336.id_kp_documentSigning, extnValue[0])
                    found = True

        self.assertTrue(found)

suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite)
